package edu.cosy.JESICA.ControllingMaster;

import edu.cosy.JESICA.Interfaces.*;
import java.util.*;
import java.rmi.*;



public class Agent implements IAgent {
	
	transient IANNFacade _facade=null;
	public ArrayList	_Way=null;
	public ArrayList	_Bag=null;
	IWorldCursor 		_cursor=null;
	int 				_nID=0;
	int 				_nMaxWay=100;
	IWorld				_world=null;
	
	
	public static final int BAGSIZE=10;
	
	//public static final int MAXWAY = 5;
	
	public Agent(int nID,IWorldCursor cursor, int nMaxWay)
	{
		_nID=nID;
		_Way=new ArrayList();
		_Bag=new ArrayList();
		_cursor=cursor;
		//cursor._world;
		_nMaxWay=nMaxWay;
	};
	
	
	public IAgentState getState() 
	{
		return new AgentState(_Way,_Bag,0,_cursor.getWorld());
	}

	public void setANN(IANNFacade ann) 
	{
		_facade=ann;
	}

	public IANNFacade getANN() 
	{
		return _facade;
	}

	public int getID() 
	{
		return _nID;
	}
	
	
	/**
	 * Binds to a world after network transmission
	 */
	public void bindWorld(IWorld wrld)
	{
		_cursor.bindWorld(wrld);

		Iterator wit=_Way.iterator();
		ArrayList newWay= new ArrayList();
		
		while(wit.hasNext())
		{
			IWorldPoint pt= (IWorldPoint)wit.next();
			pt.bindWorld(wrld);
			newWay.add(pt);
		};
		
		_Way=newWay;
		
		Iterator bit=_Bag.iterator();
		ArrayList newBag= new ArrayList();
		
		while(bit.hasNext())
		{		
			IWorldPoint pt= (IWorldPoint)bit.next();
			pt.bindWorld(wrld);
			newBag.add(pt);
		}
		
		_Bag=newBag;
		
	}

	public void performTraining(IAgent.IAgentTrainingInterrupt interrupter)
	{
		
		// Fill agents bag:
		_Bag.clear();
		
		for(int i=0; i<BAGSIZE;i++) 
		{
			_Bag.add(_cursor.getPoint());
			_Way.add(_cursor.getPoint());
			doMove();
		};
		
		
		for( int j=0 ; !interrupter.isInterrupted() && j < _nMaxWay; j++)
		{
			
			doMove();
			IWorldPoint point=_cursor.getPoint();
			_Way.add(point);
			
			
			IWorldPoint[] valid=createPointField(_Way);
			
			try
			{
	
				_facade.newNet();
				_facade.setMinimumError(0.001);
				_facade.setMaxTrainingCycles(1000);
				
				_facade.setPatternSet(createPointField(_Bag));
				_facade.performTraining(interrupter);
				double oldErr= _facade.getMSE(valid);
				int oldIndex= -1;
				
				
				
				IWorldPoint ptBackup=null;
				
				for(int i=0;i<BAGSIZE;i++)
				{
					ptBackup= (IWorldPoint)_Bag.get(i);
					
					_Bag.remove(i);
					_Bag.add(i,point);
					
					_facade.newNet();
					_facade.setPatternSet(createPointField(_Bag));
					_facade.performTraining(interrupter);
					
					double err= _facade.getMSE(valid);
									
					if(err<oldErr) oldIndex=i;
					
					_Bag.remove(i);
					_Bag.add(i,ptBackup);
	
					if(interrupter.isInterrupted()) return;
				};
					
					
				System.out.println("Agent "+ getID() +" training " + j +" finished, swapping Bagindex to" + oldIndex);
				
				if(oldIndex >=0)
				{
					_Bag.remove(oldIndex);
					_Bag.add(oldIndex,point);
				};			
			}
			catch(RemoteException e)
			{
				System.out.println(e.toString());
			};
		
		}		
		
		System.out.println("Finished Agent " + getID());
		
		return;
	}
	
	
	
	public IWorldPoint[] createPointField(ArrayList ptCol)
	{
		IWorldPoint[] points= new IWorldPoint[ptCol.size()];
		
		for(int i=0;i<points.length;i++)
		{
			points[i]=(IWorldPoint)ptCol.get(i);
		};
		
		return points;
	}
		
	
	public void doMove()
	{
		IWorldPoint pt;
		do
		{			
			MoveStep();
			pt=_cursor.getPoint();
		}
		while(!pt.hasData() /*|| !isInWay(pt,_Bag)*/);
	}
		
	
	public boolean isInWay(IWorldPoint pt, ArrayList way)
	{
		Iterator it= way.iterator();
		
		while(it.hasNext())
		{
			IWorldPoint oldpt=(IWorldPoint)it.next();
			
			if(oldpt.getX()==pt.getX() && oldpt.getY()==pt.getY()) return true;
		};
		
		return false;
	}
	
	
	public void MoveStep()	
	{
		double rnd= java.lang.Math.random()*8;
		int direction=(int)java.lang.Math.round(rnd);
		
		switch(direction)
		{
			case 0: _cursor.goWest();_cursor.goWest(); break;
			case 1: _cursor.goNW();_cursor.goNW(); break;
			case 2: _cursor.goNorth(); _cursor.goNorth(); break;
			case 3: _cursor.goNE(); _cursor.goNE(); break;
			case 4: _cursor.goEast(); _cursor.goEast(); break;
			case 5: _cursor.goSE();  _cursor.goSE(); break;
			case 6: _cursor.goSouth(); _cursor.goSouth(); break;
			case 7: _cursor.goSW(); _cursor.goSW(); break;
			default: _cursor.goNorth();_cursor.goNorth();
		};
	}		
		
	
	
	/**	
	 * Queries the neural net for classifikation of the given point
	 */
	public float queryResult(IWorldPoint pt)
	{
		try
		{
			return (float)_facade.getClassfication(pt);
		}
		catch(RemoteException e)
		{
			System.out.println(e.toString());
			return Float.NaN;
		};
	}


	class AgentState implements IAgentState
	{
		public IWorldPoint[]	_Way	={};
		public IWorldPoint[]	_Bag	={};
		public float			_Error	=0;
		public IWorldRect		_BoundingBox =null;
		
		AgentState(ArrayList way, ArrayList bag, float mse, IWorld world)
		{
			_Error=mse;
			setWay(way,world);
			setBag(bag,world);
		}
		
			
		void setWay(ArrayList way, IWorld world)
		{
			_Way= new IWorldPoint[way.size()];

			for(int i=0;i<_Way.length;i++)
			{
				_Way[i]=(IWorldPoint)way.get(i);
				//_Way[i].bindWorld(world);
			}
			
			
			if(_Way.length<1) 
			{
				_BoundingBox=null;
			}
			else
			{
				//IWorldPoint pt=_Way[0];
				//IWorld wrld= pt._world;
				
				if(world==null)
				{
					System.out.println("Error: No World in AgentState::setWay!");
					return;
				};
				
				_BoundingBox = world.getBoundingRect();
				_BoundingBox.setRect(_Way[0],_Way[0]);
				
				for(int i=0;i<_Way.length;i++)
				{
					_BoundingBox.enlargeBounding(_Way[i]);
				};
			};
			
		}
			
		void setBag(ArrayList bag, IWorld world)
		{
			_Bag= new IWorldPoint[bag.size()];

			for(int i=0;i<_Bag.length;i++)
			{
				_Bag[i]=(IWorldPoint)bag.get(i);
				//_Bag[i].bindWorld(world);
			}
		}
		
		public IWorldRect getBoundingBox()
		{
			return _BoundingBox;
		}
		
		public float getError()
		{
			return _Error;
		}


		public IWorldPoint[] getBag()
		{
			return _Bag;
		}


		public IWorldPoint [] getWay()
		{
			return _Way;
		}
		
		public void bindWorld(IWorld wrld)
		{
			for(int i=0;i<_Bag.length;i++)
			{
				_Bag[i].bindWorld(wrld);
			}
			
			for(int i=0;i<_Way.length;i++)
			{
				_Way[i].bindWorld(wrld);
			}
			
			if(_BoundingBox!=null) _BoundingBox.bindWorld(wrld);
			
		}
	}


}
