package edu.cosy.JESICA.ControllingMaster;

import edu.cosy.JESICA.Interfaces.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.*;
import java.io.*;
import java.util.*;
import java.net.*;
import edu.cosy.netgen.ann.*;
import edu.cosy.netgen.parser.*;



public class Master extends UnicastRemoteObject 
implements IControllingMaster, IControllingMasterCallbacks, IAgentFactory 
{
	
	
	// Construction
	public Master() throws RemoteException
	{
		super();	// call superclass constructor to export this object
		_slaves=new ArrayList();

	}
		
	// Main method for registering object
	public static void main(String[] args) 
	{
		try
		{
			System.out.println("Welcome to JESICA Controlling Master!");
			Master master=new Master();

			System.out.println("Master instantiated: " +master);

			Naming.rebind("/JESICA_ControllingMaster",master);
			System.out.println("Master bound to registry.");
			
		}
		catch(Exception e)
		{
			System.out.println("ERROR: Exception in slave main: "+e.toString());
		};
	}
	
	
	
	//*****
	// Controlling master implementation
	//*****	
	
	
	
	private IWorld		_world=null;  // Use World
	private ArrayList 	_slaves=null; // Array of SlaveEntry
	private int			_nAgents=0;
	private int 		_nTrainedAgents=0;
	private IManagementClientCallbacks	_cbManager=null;
	private int 		_nMaxAgentWayLength=100;

	
	
	
	/**
	 * Adds a slave that provides training power
	 * @param The host for the slave including registry entry
	 *		  e.g. "rmi://myserver.cosy.sbg.ac.at/MyWorkingSlave
	 */
	public void addSlave(String strHost) throws RemoteException 
	{
		System.out.println("Remote call: addSlave("+strHost+")");

		try
		{		
			IWorkingSlaveController slave= (IWorkingSlaveController)Naming.lookup(strHost);
			
			if(slave==null)
			{
				System.out.println("ERROR: Unable to find Working Slave on host '" +strHost +"'.");
				return;
			};
			
			slave.setWorld(_world);
			slave.setCallback(this);
			_slaves.add(new SlaveEntry(slave,strHost));
		
		}
		catch(NotBoundException e)
		{
			String strError="ERROR: Working Slave not bound on host '" +strHost +"'. Exception:"+e.toString();
			System.out.println(strError);
			return;
		}
		catch(MalformedURLException e)
		{
			String strError="ERROR: Wrong URL for new Working Slave :'" +strHost +"'. Exception:"+e.toString();
			System.out.println(strError);
			throw new RemoteException(strError);
		}
			
	}

	 /**
	  * Writes the resulting pattern file
	  */
	public void writePatternFile(String strFname) throws RemoteException 
	{
		// TODO: Add your code here
		System.out.println("Remote call: writePatternFile("+strFname+")");
		
		try
		{
			// First calculate Results:
			Result[] result= calculateResult();
			
			// create file name
			//URL url= getClass().getResource("/pattern/"+strFname);
			//String strlocalFname= url!=null ?  url.getFile() : strFname;
			System.out.println("Writing file " + strFname);
						
			// create pattern sets
			Pattern[]	ip= new Pattern[result.length];
			Pattern[]	op= new Pattern[result.length];
			
			for(int i=0;i<result.length;i++)
			{
				ip[i]= new Pattern(2);
				ip[i].setValue(0,result[i]._point.getX());
				ip[i].setValue(1,result[i]._point.getY());
				
				op[i]= new Pattern(1);
				op[i].setValue(0,result[i]._result);
			}
			PatternSet ps= new PatternSet(ip,op);
				
			// open parser
			AnnSNNSParser parser= new AnnSNNSParser();
		
			if(!parser.exportPatterns(new File(strFname),ps))
			{
				System.out.println("Error writing pattern file " +strFname);
			}
			else
			{			
				System.out.println("Written " + result.length +" patterns to file " +strFname);
			};
			
		}
		catch(Exception e)
		{
			System.out.println(e.toString());
		};

	}
	
	 /**
	  * Writes the network of a given agent to a file
	  */
	public void writeNetFile(int nAgentID, String strFname) throws RemoteException 
	{
		// TODO: Add your code here
		System.out.println("Remote call: writeNetFile("+nAgentID+ ","+strFname+")");
	}


	 /**
	  * Writes each agents way to a pattern file
	  */
	 public void writeAgentWays(String strFilePrefix, String strFileSuffix) throws RemoteException
	 {
	 	try
	 	{
			// get list of all Agents
			ArrayList agents= getAgentArrayList();
			Iterator it=agents.iterator();
			
			while(it.hasNext())
			{
				IAgent agent=(IAgent)it.next();
				IAgentState state=agent.getState();
				
				IWorldPoint[] way= state.getWay();
				
				// create pattern sets
				Pattern[]	ip= new Pattern[way.length];
				Pattern[]	op= new Pattern[way.length];
				
				for(int i=0;i<way.length;i++)
				{
					ip[i]= new Pattern(2);
					ip[i].setValue(0,way[i].getX());
					ip[i].setValue(1,way[i].getY());
					
					op[i]= new Pattern(1);
					op[i].setValue(0,way[i].getData());
				}
				
				PatternSet ps= new PatternSet(ip,op);
					
				// open parser
				AnnSNNSParser parser= new AnnSNNSParser();
			
				String strFname= strFilePrefix+"Agent"+agent.getID()+strFileSuffix;
			
				if(!parser.exportPatterns(new File(strFname),ps))
				{
					System.out.println("Error writing waypoint file " +strFname);
				}
				else
				{			
					System.out.println("Written " + way.length +" waypoints to file "+ strFname);
				};
			}
			
	 		
		}
		catch(Exception e)
		{
			System.out.println(e.toString());
		};
	 		
	 }


	 /**
	  * Reads a pattern file and creates the current world
	  */
	public void readPatternFile(String strFname) throws RemoteException 
	{
		// TODO: Add your code here
		System.out.println("Remote call: readPatternFile("+strFname+")");
		
		try
		{
			URL url= getClass().getResource("/pattern/"+strFname);
			
			String strlocalFname= url!=null ?  url.getFile() : strFname;
			System.out.println("Parsing file " + strlocalFname);
			
			AnnSNNSParser parser= new AnnSNNSParser();
			PatternSet pattern = parser.importPatterns(new File(strlocalFname));
			
			Pattern[] inp=pattern.getInPatterns();
			Pattern[] outp=pattern.getOutPatterns();
			
			ArrayList patternList = new ArrayList();
			
			for(int i=0;i<inp.length;i++)
			{
			 	Pattern ip= inp[i];
			 	Pattern op= outp[i];
			 	
			 	if(ip.getSize()!=2)
			 	{
			 		System.out.println("Invalid input pattern: Dimension should be 2, but was " + ip.getSize());
			 		return;
			 	};
			 	if(op.getSize()!=1)
			 	{
			 		System.out.println("Invalid output pattern: Dimension should be 1, but was " + op.getSize());
			 		return;
			 	};
			 	
			 	DataPoints dp= new DataPoints();
			 	dp.x=(int)ip.getValue(0);
			 	dp.y=(int)ip.getValue(1);
			 	dp.d=(float)op.getValue(0);
			 	
			 	patternList.add(dp);
			};
			
			System.out.println("Read " + patternList.size() +" patterns from file.");
			 	
			// create pattern field
			
			DataPoints[] dpts= new DataPoints[patternList.size()]; 
			
			for(int i=0; i<patternList.size();i++)
			{
				dpts[i]=(DataPoints)patternList.get(i);
			};
			
			// Set new world
			DiscreteWorld wrld= new DiscreteWorld();
			wrld.setData(dpts);
			
			setWorld(wrld);
			
		}
		catch(Exception e)
		{
			System.out.println(e.toString());
		};
	
	}
	
	
	public void setWorld(IWorld world)
	{
		_world=world;
		System.out.println("Master::setWorld");
		_world.DebugWorld();
	}


	 /**
	  * Create initial number of Agents
	  */
    public void setInitialAgents(int nAgents)  throws RemoteException
    {
		System.out.println("Remote call: setInitialAgents("+nAgents+")");
		
		int nSlaves= _slaves.size();
		
		if(nSlaves==0) 
		{
			String strError="No Slaves available";
			System.out.println(strError);
			throw new RemoteException(strError);
		};
		
		int nApS=nAgents/nSlaves;
		int nAPlus=nAgents%nSlaves;
		
		Iterator it=_slaves.iterator();

		IWorldRect rect= _world.getBoundingRect();
		IWorldPoint nw= rect.getNWPoint();
		IWorldPoint sw= rect.getSEPoint();
				
		double Xdist=	(double)rect.getWEDistance();
		double Ydist=	(double)rect.getNSDistance();
		
		IWorldCursor cursor = _world.getCursor();

		
		// Create agents per slave
		while(it.hasNext())
		{
			SlaveEntry slave=(SlaveEntry)it.next();
			
			int nAg=nApS;
			if(nAPlus!=0)
			{
				nAg+=nAPlus;
				nAPlus=0;
			};
			
			for(int i=0;i<nApS;i++)
			{
				
				int x= nw.getX() + (int)Math.round(Math.random()*Xdist);
				int y= nw.getY() + (int)Math.round(Math.random()*Ydist);

				cursor.gotoPoint(x,y);

				IAgent agent= createAgent(cursor.getPoint());
				
				slave.addAgent(agent);
				slave.getSlave().addAgent(agent);
			};
		};
			
		
	}

	public void setMaxAgentWayLength(int nMaxWay) throws RemoteException
	{
		_nMaxAgentWayLength=nMaxWay;
	}


	 /**
	  * Starts training of all current agents
	  */
	public void startTraining(IManagementClientCallbacks cbManager) throws RemoteException 
	{
		// TODO: Add your code here
		System.out.println("Remote call: startTraining("+cbManager+")");
		try
		{
			_cbManager=cbManager;
			
			
			Iterator it = _slaves.iterator();
			while(it.hasNext())
			{
				SlaveEntry slave=(SlaveEntry)it.next();
				
				if(slave==null || slave.getSlave()==null)
				{
					String strError="ERROR: Slave was null!";
					System.out.println(strError);
					throw new RemoteException (strError);
				}
				
				
				//slave.getSlave().setCallback(this);
				
				Iterator ait=slave.getAgents();
				while(ait.hasNext())
				{
					IAgent agent= (IAgent) ait.next();
					if(agent==null)
					{
						String strError="ERROR: Agent was null!";
						System.out.println(strError);
						throw new RemoteException (strError);
					}

					System.out.println("Start Training of Agent " + agent.getID());
					slave.getSlave().trainAgent(agent.getID());
					System.out.println("Started Training of Agent " + agent.getID());
				};
				
			};
			
			System.out.println("Start of training finished...");
			
		}
		catch(Exception e)
		{
			System.out.println(e.toString());
		};
	}



	 /**
	  * Releases all agents 
	  */
	public void Release() throws RemoteException 
	{
		System.out.println("Remote call: Release()");
		_world=null;
		
		Iterator it = _slaves.iterator();
		while(it.hasNext())
		{
			SlaveEntry slave=(SlaveEntry)it.next();
			if(slave==null)
			{
				String strError="ERROR: Slave was null!";
				System.out.println(strError);
				throw new RemoteException (strError);
			}
			
			System.out.println("Release Agents of Slave " + slave.getHost());
			
			Iterator ait=slave.getAgents();
			while(ait.hasNext())
			{
				IAgent agent= (IAgent) ait.next();

				if(agent==null)
				{
					String strError="ERROR: Agent was null!";
					System.out.println(strError);
					throw new RemoteException (strError);
				}
				System.out.println("Remove Agent " + agent.getID());
				slave.getSlave().removeAgent(agent.getID());
			};
			
			slave.removeAgents();
		};
		
		_nAgents=0;
		_nTrainedAgents=0;
	}
	
	
	public IAgentFactory getAgentFactory()
	{
		return this;
	}

	/**
	 * Creates a new Agent
	 */
	public IAgent createAgent(IWorldPoint pt) throws RemoteException {
		// TODO: Add your code here
		_nAgents++;
		IWorldCursor cs= _world.getCursor();
		cs.gotoPoint(pt.getX(),pt.getY());
		return new Agent(_nAgents,cs,_nMaxAgentWayLength);
	}


	/** Callback when a Agent finished training
	 *
	 */
	public void AgentTrainingFinished(int nAgentID) throws RemoteException 
	{
		// TODO: Add your code here
		System.out.println("Remote call: AgentTrainingFinished("+nAgentID+")");
		
		_nTrainedAgents++;
		
		
		/*
		if(_nAgents==_nTrainedAgents)
		{
			calculateResult();
		};

		*/
		
		if(_cbManager!=null)
		{
			_cbManager.AgentTrainingFinished(nAgentID);
			
			if(_nAgents==_nTrainedAgents)
			{
				_cbManager.SetResult(null);
			};
		};
		
	}
	

	/**
	 * Calculates the result
	 */

	public Result[] calculateResult()
	{
		try
		{
			
			//System.out.println("calculateResult: CheckPoint1");
			
			// first get all Agents back from the slaves
			Iterator sit= _slaves.iterator();
			
			while(sit.hasNext())
			{
				SlaveEntry slave=(SlaveEntry)sit.next();
				slave.refreshAgents(_world);
			}

			//System.out.println("calculateResult: CheckPoint2");
			
			
			// get list of all Agents
			ArrayList agents= getAgentArrayList();
			
			IWorldRect worldRect=_world.getBoundingRect();
			
			ArrayList results= new ArrayList();
			// drive through the world and collect all data points
			
			IWorldCursor cursor=_world.getCursor();
			cursor.gotoPoint(worldRect.getNWPoint());
			
			int xd=worldRect.getWEDistance();
			int yd=worldRect.getNSDistance();
			
			// move cascading through the world...
//			System.out.println("calculateResult: CheckPoint3");
			
			for(int k=0;k<=yd;k++)
			{				
				for(int i=0;i<=xd;i++)
				{
					IWorldPoint pt=cursor.getPoint();
					
					float res=calculateResultPoint(pt,agents.iterator());
					
					
					results.add(new Result(pt,res));
					
					if(k%2==0)
					{
						cursor.goEast();
					}
					else
					{
						cursor.goWest();
					};
				};
				
				cursor.goSouth();
			};
			
//			System.out.println("calculateResult: CheckPoint4");
			
			
			Result[] points= new Result[results.size()];
			
			Iterator rit=results.iterator();
			
			for(int i=0; i<points.length;i++)
			{
				points[i]=(Result)results.get(i);
			};

			return points;
			
			
		}
		catch(RemoteException e)
		{
			System.out.println(e.toString());
		};
		
		return new Result[0];
			
	}


	/**
	 * Fills a world point with results
	 */
	 public float calculateResultPoint(IWorldPoint pt, Iterator it)
	 {
	
		//System.out.println("calculateResultPoint: CheckPoint1; it " + it.toString());

	 	// first search the nearest Agent;
	 	IAgent agent=null;
	 	long dist = java.lang.Long.MAX_VALUE;
	 	
	 	while(it.hasNext())
	 	{
	 		IAgent ag=(IAgent)it.next();
	 		
			//System.out.println("calculateResultPoint: CheckPoint1.1; ag " +ag.toString());
	 		
	 		IAgentState state=ag.getState();
	 		
			//System.out.println("calculateResultPoint: CheckPoint2");
	 		
	 		if(state.getBoundingBox().isInRect(pt)) 
	 		{
	 			agent=ag;
	 			break;
	 		};
	 		
			//System.out.println("calculateResultPoint: CheckPoint3");
	 		
	 		long ld=calculateDistance(pt,state.getBoundingBox());
	 		if(ld<dist)
	 		{
	 			agent=ag;
	 			dist=ld;
	 		};
	 		
	 		
			//System.out.println("calculateResultPoint: CheckPoint4");
	 		
	 	};
	 	
	 	
	 	if(agent==null)
	 	{
	 		System.out.println("ERROR: No agents found to query data!");
	 		return 0;
	 	};
	 	
		//System.out.println("calculateResultPoint: CheckPoint5");
	 	
	 	float res= agent.queryResult(pt);

		//System.out.println("calculateResultPoint: CheckPoint6");
		
		return res;
	 	
	 }
	 
	 	
	 /**
	  * Calculates the distance between two points
	  */	 
	 public long calculateDistance(IWorldPoint point, IWorldRect rect)
	 {
	 	IWorldPoint center=rect.getCenterPoint();
	 	
	 	int Xdist=java.lang.Math.abs(point.getX()-center.getX());
	 	int Ydist=java.lang.Math.abs(point.getY()-center.getY());
	 	long distance=Xdist*Xdist+Ydist*Ydist;
	 	return distance;
	 }
	 
	/**
	 * Returns a list of all Agents
	 */
	public ArrayList getAgentArrayList()
	{
		ArrayList agents= new ArrayList();
		
		Iterator it=_slaves.iterator();
		while(it.hasNext())
		{
			Iterator ait=((SlaveEntry)it.next()).getAgents();
			while(ait.hasNext())
			{
				agents.add(ait.next());
			};
		};
		return agents;
	}

	
	// Search an agent
	IAgent searchAgent(int nAgentID)
	{
		IAgent agent=null;
		
		Iterator it=_slaves.iterator();
		
		while(it.hasNext())
		{
			SlaveEntry slave=(SlaveEntry)it.next();
			agent=slave.searchAgent(nAgentID);
			if(agent!=null) return agent;
		};
		
		return null;
	}

	// Search an agent's slave
	SlaveEntry searchAgentSlave(int nAgentID)
	{
		IAgent agent=null;
		
		Iterator it=_slaves.iterator();
		
		while(it.hasNext())
		{
			SlaveEntry slave=(SlaveEntry)it.next();
			agent=slave.searchAgent(nAgentID);
			if(agent!=null) return slave;
		};
		
		return null;
	}

	
	// Class for an Entry in the Slaves list
	private class SlaveEntry
	{
		ArrayList					_agents=null;
		IWorkingSlaveController   	_slave=null;
		String						_strHost="";
		
		SlaveEntry(IWorkingSlaveController slave, String strHost)
		{
			_slave=slave;
			_agents= new ArrayList();
			_strHost=strHost;
		}
		
		IWorkingSlaveController getSlave(){
			return _slave;
		}
		
		String getHost(){
			return _strHost;
		}
		
		
		void addAgent(IAgent agent)
		{
			_agents.add(agent);
		}
		
		Iterator getAgents()
		{
			return _agents.iterator();
		}
		
		void removeAgents()
		{
			_agents.clear();
		}
		
		IAgent searchAgent(int nAgentID)
		{
			Iterator it=getAgents();
			
			while(it.hasNext())
			{
				IAgent agent= (IAgent)it.next();
				
				if(agent.getID()==nAgentID) return agent;
			};
			
			return null;
		}
		
		
		/**
		 * Refreshes agents from the slave
		 */
		void refreshAgents(IWorld world) throws RemoteException
		{
			ArrayList newAgents= new ArrayList();
			
			Iterator it=getAgents();
			
			// get Agents from slave
			while(it.hasNext())
			{
				IAgent oldAgent=(IAgent)it.next();
				IAgent newAgent=_slave.getAgent(oldAgent.getID());
				
				IANNFacade facade=_slave.getFacade(oldAgent.getID());
				
				System.out.println("Set facade of Agent" + newAgent.getID() + " to " + facade.toString());
				
				newAgent.bindWorld(world);
				newAgent.setANN(facade);
				
				newAgents.add(newAgent);
				
			};
			
			// replace old agents with new ones
			_agents=newAgents;
		}
			
			
	}
	
	public class Result
	{
		IWorldPoint _point;
		float		_result;
		
		Result(IWorldPoint pt, float res)
		{
			_point=pt;
			_result=res;
		}
			
	}

	
}
