package edu.cosy.JESICA.Interfaces;
import java.rmi.*;

/**
 * Interface for central intelligence component
 * @author taschaue
 */
public interface IControllingMaster extends Remote
{
	/**
	 * Adds a slave that provides training power
	 * @param The host for the slave including registry entry
	 *		  e.g. "rmi://myserver.cosy.sbg.ac.at/MyWorkingSlave
	 */
	 public void addSlave(String strHost) throws RemoteException;
	 
	 
	 /**
	  * Writes the resulting pattern file
	  */
	 public void writePatternFile(String strFname) throws RemoteException;
	 
	 /**
	  * Writes the network of a given agent to a file
	  */
	 public void writeNetFile(int nAgentID, String strFname) throws RemoteException;
	 
	 
	 /**
	  * Writes each agents way to a pattern file
	  */
	 public void writeAgentWays(String strFilePrefix, String strFileSuffix) throws RemoteException;
	 
	 /**
	  * Reads a pattern file and creates the current world
	  */
	 public void readPatternFile(String strFname) throws RemoteException;
	 
	 /**
	  * Create initial number of Agents
	  */
	 public void setInitialAgents(int nAgents) throws RemoteException;

	 /**
	  * Starts traing of all current agents
	  */
	 public void startTraining(IManagementClientCallbacks cbManager) throws RemoteException;
	 
	 /**
	  * Set max length of agent way
	  */
	 public void setMaxAgentWayLength(int nMaxWay) throws RemoteException;
	  
	 /**
	  * Releases all agents 
	  */
	 public void Release() throws RemoteException;
	
}
