package edu.cosy.JESICA.ManagementClient;
import edu.cosy.JESICA.Interfaces.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.*;
import java.io.*;
import java.util.*;


/**
 * Console Management client
 */
public class ConsoleManager implements IManagementClientCallbacks
{
	


	public static void main(String[] args) 
	{
		// Export the callback object
		System.out.println("Welcome to JESICA Management Client!");
		ConsoleManager mgr=new ConsoleManager();
		mgr.run();
		System.out.println("Manager session closed. See you ;-)");
		
	}
		
		
	public void run()
	{
		try
		{
			System.out.println("Starting...");
			
			// Export object
			UnicastRemoteObject.exportObject(this);
			
			System.out.println("Looking for Controlling Master...");
			
			// Get local Controlling master
			String strMaster = "rmi://127.0.0.1/JESICA_ControllingMaster";
			IControllingMaster master= (IControllingMaster)Naming.lookup(strMaster);
			
			if(master==null)
			{
				System.out.println("ERROR: Unable to find Controlling Master on host '" +strMaster +"'.");
				return;
			};

			// Creating world			
			System.out.println("Reading XOR-Pattern file...");
			master.readPatternFile("xor.pat");
			
			
			// Adding local slave
			System.out.println("Adding local slave...");
			String strSlave = "rmi://127.0.0.1/JESICA_WorkingSlave";
			master.addSlave(strSlave);
			
			// Do training

			int nMaxWay=20;
			int nAgent=4;
			
			try
			{
				
			   	FileInputStream in = new FileInputStream(new File("JESICA.properties"));
				Properties prop= new Properties();
				prop.load(in);
				nMaxWay=Integer.decode(prop.getProperty("MaxAgentWayLength")).intValue();
				nAgent=Integer.decode(prop.getProperty("InitialAgents")).intValue();
			}
			catch(IOException e)
			{
				System.out.println("Error reading properties, using default values. " + e.toString());
			};

			
			
			System.out.println("Setting up agents; agents=" +nAgent+" , steps= "+nMaxWay);
			master.setMaxAgentWayLength(nMaxWay);
			master.setInitialAgents(nAgent);
			
			System.out.println("Start Training...");
			m_bAllAgentTrainingFinished=false;
			master.startTraining(this);
			
			while(!m_bAllAgentTrainingFinished) 
			{
				System.out.print('.');
				Thread.sleep(1000); // Wait for finished
			};
			
			System.out.println("Training finished!");
			
			System.out.println("Writing result file...");
			master.writePatternFile("xor.res");
			
			System.out.println("Writing agent waypoints...");
			master.writeAgentWays("waypoints_",".way");

			
			System.out.println("Releasing slaves and master...");
			master.Release();
			master=null;
						
		}
		catch(Exception e)
		{
			System.out.println("Exception in Management Client: " +e.toString());
		}
	}


	protected boolean 	m_bAllAgentTrainingFinished=false;
	
	/** 
	 * Notify that a Agent has finished training
	 */
	 public void AgentTrainingFinished(int nAgentID) throws RemoteException
	 {
	 	System.out.println("Agent " + nAgentID + " has finished trainig.");
	 	
	 }
	 
	 /** 
	  * Sets the result ao trainig
	  */
	 public void SetResult(IWorldPoint[] pts) throws RemoteException
	 {
	 	
	 	System.out.println("System results received...");
	 	m_bAllAgentTrainingFinished=true;
	 }
}
