package edu.cosy.JESICA.Test;
import edu.cosy.JESICA.ControllingMaster.*;
import edu.cosy.JESICA.Interfaces.*;
import junit.framework.*;


public class TestMaster extends TestCase {
	
	
	/**
	 * Default constructor for JUnit test case
	 */
	public TestMaster(String arg1)
	{
		super(arg1);
	}
	
	
	/**
	 * Main method running the test case
	 */
	public static void main(String[] args) 
	{
		junit.textui.TestRunner.run(suite());
	}	
	
	/**
	 * Helper function for getting the class of the test case
	 */
	public static Test suite()
	{
		return new TestSuite(TestMaster.class);
	}


	private static final int _worldSize=100;
	
	
	/**
	 * Helper function creating a discrete world
	 */
	public DiscreteWorld createWorld()
	{
		DataPoints[] pts =  new DataPoints[_worldSize];
		DiscreteWorld world= new DiscreteWorld();
		
		for(int i=0;i<pts.length;i++)
		{
			DataPoints p=new DataPoints();
			
			assertNotNull("i="+i+", pts.length="+pts.length,p);
			
			p.x=i;
			p.y=i;
			p.d=i;
			
			pts[i]=p;
			
		}
		
		world.setData(pts);
		return world;
	}

	/**
	 * Helper function comparing two world points
	 */
	public void assertPoint(String str, IWorldPoint pt1, IWorldPoint pt2)
	{
		assertEquals(str+" [x]",pt1.getX(),pt2.getX());
		assertEquals(str+" [y]",pt1.getY(),pt2.getY());
	}
	
	public void testResultCreation() throws Exception
	{
		
		Master master= new Master();
		DiscreteWorld world = createWorld();
		//master.setWorld(world);
		master.readPatternFile("xor.pat");
		
		Master.Result[] result=master.calculateResult();
		
		assertEquals("Result size",4,result.length);

		DiscreteWorldPoint pt1= new	DiscreteWorldPoint(world,5,5);
		DiscreteWorldPoint pt2= new	DiscreteWorldPoint(world,35,35);
		DiscreteWorldPoint pt3= new	DiscreteWorldPoint(world,5,35);
		DiscreteWorldPoint pt4= new	DiscreteWorldPoint(world,35,5);
		DiscreteWorldPoint pt5= new	DiscreteWorldPoint(world,20,35);
		DiscreteWorldPoint nw= new 	DiscreteWorldPoint(world,10,30);
		DiscreteWorldPoint se= new 	DiscreteWorldPoint(world,30,10);
		
		DiscreteWorldRect rect= (DiscreteWorldRect)world.getBoundingRect();
		
		rect.setRect(nw,se);
		
		assertEquals("Distance calc",450,master.calculateDistance(pt1,rect));
		assertEquals("Distance calc",450,master.calculateDistance(pt2,rect));
		assertEquals("Distance calc",450,master.calculateDistance(pt3,rect));
		assertEquals("Distance calc",450,master.calculateDistance(pt4,rect));
		assertEquals("Distance calc",225,master.calculateDistance(pt5,rect));
		
		
		master.writePatternFile("xor.res");
			
			
	}
	
	
	public void testAgent() throws Exception
	{
		
		Master master= new Master();
		DiscreteWorld world = createWorld();
		//master.setWorld(world);
		master.readPatternFile("xor.pat");
		
		DiscreteWorldPoint pt1= new	DiscreteWorldPoint(world,5,5);

		Agent agent= (Agent)master.createAgent(pt1);
		
		
		DiscreteWorldPoint pt2= new	DiscreteWorldPoint(world,35,35);
		DiscreteWorldPoint pt3= new	DiscreteWorldPoint(world,5,35);
		DiscreteWorldPoint pt4= new	DiscreteWorldPoint(world,35,5);
		DiscreteWorldPoint pt5= new	DiscreteWorldPoint(world,20,35);
		DiscreteWorldPoint nw= new 	DiscreteWorldPoint(world,10,30);
		DiscreteWorldPoint se= new 	DiscreteWorldPoint(world,30,10);
		

		DiscreteWorldPoint[] pts1={pt1,pt2,pt3,pt4,pt5,nw,se};
		DiscreteWorldPoint[] pts2={pt1,pt2,pt3,pt4,pt5};
		
		agent._Bag.add(pt1);
		agent._Bag.add(pt2);
		agent._Bag.add(pt3);
		agent._Bag.add(pt4);
		agent._Bag.add(pt5);

		agent._Way.add(pt1);
		agent._Way.add(pt2);
		agent._Way.add(pt3);
		agent._Way.add(pt4);
		agent._Way.add(pt5);
		agent._Way.add(nw);
		
		
		IAgentState state = agent.getState();
		
		
	}

}
