package edu.cosy.JESICA.Test;
import edu.cosy.JESICA.ControllingMaster.*;
import edu.cosy.JESICA.Interfaces.*;
import edu.cosy.JESICA.WorkingSlave.*;
import junit.framework.*;


public class TestWorld extends TestCase{
	
	
	/**
	 * Default constructor for JUnit test case
	 */
	public TestWorld(String arg1)
	{
		super(arg1);
	}
	
	
	/**
	 * Main method running the test case
	 */
	public static void main(String[] args) 
	{
		junit.textui.TestRunner.run(suite());
	}	
	
	/**
	 * Helper function for getting the class of the test case
	 */
	public static Test suite()
	{
		return new TestSuite(TestWorld.class);
	}

	
	
	/**
	 * Helper function creating a discrete world
	 */
	public DiscreteWorld createWorld()
	{
		DataPoints[] pts =  new DataPoints[100];
		DiscreteWorld world= new DiscreteWorld();
		
		for(int i=0;i<pts.length;i++)
		{
			DataPoints p=new DataPoints();
			
			assertNotNull("i="+i+", pts.length="+pts.length,p);
			
			p.x=i;
			p.y=i;
			p.d=i;
			
			pts[i]=p;
			
		}
		
		world.setData(pts);
		return world;
	}

	/**
	 * Helper function comparing two world points
	 */
	public void assertPoint(String str, IWorldPoint pt1, IWorldPoint pt2)
	{
		assertEquals(str+" [x]",pt1.getX(),pt2.getX());
		assertEquals(str+" [y]",pt1.getY(),pt2.getY());
	}
	
	/**
	 * Test method: test JFroehlichFacade
	 *//*	
	public void testJNNFacade() {
    	JFroehlichANNFacade JNNF = new JFroehlichANNFacade();
    	DiscreteWorld world=createWorld();
    	
    	DiscreteWorldPoint[] patternSet = { new DiscreteWorldPoint(world,1,2),
    										new DiscreteWorldPoint(world,3,4),    								  	
    								  };
	
		//JNNF.setPatternSet(patternSet);
		
    	JNNF.setLearningRate((float)0.25);
    	JNNF.setMaxTrainingCycles(1000);
    	JNNF.setMinimumError((float)0.00005);
    	//JNNF.setDisplayStep(10); 
    	//JNNF.performTraining();
	}*/
	
	/**
	 * Test method: test jaNetFacade
	 */	
	public void testJaNetFacade() throws Exception
	{
		jaNetFacade jaNetF = new jaNetFacade();
		
    	DiscreteWorld world=createWorld();
    	
    	DiscreteWorldPoint[] patternSet = { new DiscreteWorldPoint(world,1,1),
    										new DiscreteWorldPoint(world,3,3),
    										new DiscreteWorldPoint(world,5,5),
    										new DiscreteWorldPoint(world,7,7),
    										new DiscreteWorldPoint(world,9,9),
    										new DiscreteWorldPoint(world,11,11),
    										new DiscreteWorldPoint(world,12,12),
    								  };
		
		jaNetF.setPatternSet(patternSet);
		jaNetF.setMinimumError(0.00000001);
		jaNetF.setMaxTrainingCycles(10000000);
		jaNetF.performTraining();
		
	}
	
	
	/**
	 * Test method: test cursor moves
	 * (Automatically called by JUnit)
	 */
	public void testCursorMovement() throws Exception
	{
	
		DiscreteWorld world=createWorld();
	
		DiscreteWorldCursor cs=(DiscreteWorldCursor)world.getCursor();
		
		cs.gotoPoint(5,5);
		
		cs.goNorth();
		assertPoint("Move North",new DiscreteWorldPoint(world,5,6),cs.getPoint());
		cs.goSouth();
		assertPoint("Move South",new DiscreteWorldPoint(world,5,5),cs.getPoint());
		cs.goEast();
		assertPoint("Move East",new DiscreteWorldPoint(world,6,5),cs.getPoint());
		cs.goWest();
		assertPoint("Move West",new DiscreteWorldPoint(world,5,5),cs.getPoint());
		cs.goNE();
		assertPoint("Move NE",new DiscreteWorldPoint(world,6,6),cs.getPoint());
		cs.goSW();
		assertPoint("Move SW",new DiscreteWorldPoint(world,5,5),cs.getPoint());
		cs.goNW();
		assertPoint("Move NW",new DiscreteWorldPoint(world,4,6),cs.getPoint());
		cs.goSE();
		assertPoint("Move SE",new DiscreteWorldPoint(world,5,5),cs.getPoint());
		
	}
	
	
	/**
	 * Test method: test bounding functions of rectangle
	 * (Automatically called by JUnit)
	 */
	public void testRect() throws Exception
	{
		DiscreteWorld world=createWorld();
		
		IWorldRect wbr=world.getBoundingRect();
		
		DiscreteWorldPoint ptNW= new DiscreteWorldPoint(world,0,99);
		DiscreteWorldPoint ptSE= new DiscreteWorldPoint(world,99,0);
		
		DiscreteWorldPoint ptIn= new DiscreteWorldPoint(world,10,10);
		DiscreteWorldPoint ptOut1= new DiscreteWorldPoint(world,-10,10);
		DiscreteWorldPoint ptOut2= new DiscreteWorldPoint(world,4,1000);
		
		
		assertPoint("NW-Point",ptNW,wbr.getNWPoint());
		assertPoint("SE-Point",ptSE,wbr.getSEPoint());
		
		assert("Point In",wbr.isInRect(ptIn));
		assert("Point Out1",!wbr.isInRect(ptOut1));
		assert("Point Out2",!wbr.isInRect(ptOut2));
		
		assertEquals("NS-Distance",99,wbr.getNSDistance());
		assertEquals("WE-Distance",99,wbr.getWEDistance());
		
		
		
	}
	
}




























