/*
 * JaNetFacade.java
 *
 * Created on 02. Mai 2001, 23:42
 */

package edu.cosy.JESICA.WorkingSlave;

import edu.cosy.JESICA.Interfaces.*;
import java.io.*;
import jaNet.backprop.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.*;


/**
 *
 * @author  mjoellnir
 * @version
 */
public class jaNetFacade implements IANNFacade{
		
    private		double[][]  _inputPattern;
    private		double[][]  _outputPattern;
    
	// the backpropagation net
	private 	BPN		_bpn;
	
	// Activation functions for jaNet
	private String _activationsFunctions[] = 
		{"jaNet.backprop.Sigmoidc","jaNet.backprop.Sigmoidc"};
	
	
	// Error of the net
	private		double	_nError = 1000000;
	
	// Maximum training cycles
	private		int		_nMaxTrainingCycles = 10000;
	
	// Minimum error for the net
	private		double	_nMinimumError = 0.001;
	
	// Momentum
	private		double	_nMomentum = 0.3;
	
	// Learning rate
	private		double  _nLearningRate = 0.5;
	
	// amount of input layer neurons
	private		int		_nInputLayerNeurons = 2;
	
	// amount of hidden layer / optional
	private		int		_nHiddenLayerNeurons = 4;
	
	// amount of output layer - dependent on patternfile
	private		int		_nOutputLayerNeurons = 1;
	
	// layer definition
	private		int[]   _layerDef;
    
   	public jaNetFacade() 
    {
    	//System.out.println("jaNetFacade::jaNetFacade("+param+")");
    	
    	try
    	{
			newNet();
			
			// Export this object
			UnicastRemoteObject.exportObject(this);
		}
		catch(RemoteException e)
		{
			System.out.println(e.toString());
		};
    }
    
	
	/**
	 * Create a new net and release a old one if it exists
	 */
	public void newNet()  throws RemoteException {
    	// setup layer definition for jaNet
    	// hidden layer is optional
    	if( _nHiddenLayerNeurons == 0 ) {
    		_layerDef = new int[] { _nInputLayerNeurons, 
    					  			_nOutputLayerNeurons
    							  };
    	}
    	else {    		
    		_layerDef = new int [] { _nInputLayerNeurons, 
    					  			 _nHiddenLayerNeurons,
    		              			 _nOutputLayerNeurons
    							   };    		
    	}
    	try {    		
    		_bpn = new BPN( _layerDef, _activationsFunctions);
    		// set standard learning parameters
    		_bpn.setLearningRate(_nLearningRate);
    		_bpn.setMomentum(_nMomentum);    		
    		
    	} catch(BPNException e) {
    		System.out.println("Error while creating Backpropagation net: " + e.toString() );
    	}
    	
    	//System.out.println("Finished creating NN...");		
	}

	/**
	 * Sets the current pattern set for training
	 */
	public void  setPatternSet(IWorldPoint[] pattern)  throws RemoteException {
		if( pattern == null ) {
			System.out.println("Error, pattern set can't be <null> !");
			return;
		}
		
		_inputPattern  = new double[pattern.length][_nInputLayerNeurons];
		_outputPattern = new double[pattern.length][_nOutputLayerNeurons];
		
		for(int i=0; i < pattern.length; i++) {
			// set input pattern array for jaNet
			_inputPattern[i][0] = pattern[i].getX();
			_inputPattern[i][1] = pattern[i].getY();
			// set output pattern array for jaNet
			if( pattern[i].hasData() ) {
				_outputPattern[i][0] = pattern[i].getData();
				if( _outputPattern[i][0] < 0.9 ) System.out.println("Found zero pattern.");
			}
			else {
				_outputPattern[i][0] = 0;
				System.out.println("Error, no data found in WorldPoint!");
			}
		}
		
		//System.out.println("Completed setting patterns...");
	}
	
	/**
	 * Performs classification on a single point
	 */
	public double getClassfication(IWorldPoint point)  throws RemoteException {
		
		double[] output;
				
		if( point == null ) {
			System.out.println("Can't classify Point <null> !");
			return -1;
		}
		try {
			double[] input = { point.getX(), point.getY() } ;
			_bpn.propagate(input);
			
			// Fetch output result from the network
			output = _bpn.getOutputVector();
						
		}catch(BPNException e) {
			System.out.println("An error occured while classifing: " + e.toString() );
			return -1;
		}
		return output[0];
	}
	
	/**
	 * Performs training of the net
	 */
	public void  performTraining(IAgent.IAgentTrainingInterrupt interrupter)  throws RemoteException {
		
		 double nError = java.lang.Double.MAX_VALUE;
		
		if( _inputPattern == null || _outputPattern == null ) {
			System.out.println("Error can't train net without any input/output patterns!");
			return;
		}
		
		try {		
			for( int nCycles = 0; 
			     !interrupter.isInterrupted() && nCycles < _nMaxTrainingCycles 
			     && nError > _nMinimumError; 
			     nCycles++) {
			     	
				 // chose at random an input pattern
				 int p = (int)(Math.random()*(double)_inputPattern.length);			     	
				 
				 // learn the input pattern in order to reproduce
				 // corresponding target
				 _bpn.learn(_inputPattern[p],_outputPattern[p]);
				
				 // compute global error making the sum of errors of all patterns			
			     nError = getCurrentError();	
				
			}
	   } catch(BPNException e) {
			System.out.println("Error while training net: " + e.toString());
	   }
	   _nError = nError;
	}
	
	/**
	 * compute global error making the sum of errors of all patterns
	 */
	 
	private double getCurrentError() throws BPNException{
		double error = 0;
		for(int i=0; i<_inputPattern.length; i++){
			_bpn.propagate(_inputPattern[i]);
			error += 0.5 * (_outputPattern[i][0] - _bpn.getOutputVector()[0])*(_outputPattern[i][0] - _bpn.getOutputVector()[0]);
		}		
		return error;
	}

	/**
	 * compute MSE
	 */
	 
	public double getMSE(IWorldPoint[] validationPattern)  throws RemoteException {		
	
		double [][] valPattern = createPatternSet(validationPattern);
		if( valPattern == null ) {
			System.out.println("Error while creating validation pattern set!");
			return -1;
		}
		
		double error = 0;
		
		try {
			for(int i=0; i<valPattern.length; i++){
				_bpn.propagate(valPattern[i]);
				error += 0.5 * ( validationPattern[i].getData() - _bpn.getOutputVector()[0])*(validationPattern[i].getData() - _bpn.getOutputVector()[0]);
			}		
		} catch(BPNException e) {
			System.out.println("Error while calculating MSE for validation pattern !");
			return -1;
		}
		return error;
	}
	
	/**
	 * Create Patternset out of Wolrdpoints
	 */
	public double[][]  createPatternSet(IWorldPoint[] pattern)  {
		if( pattern == null ) {
			System.out.println("Error, pattern set can't be <null> !");
			return null;
		}
		
		double[][] validationPattern  = new double[pattern.length][_nInputLayerNeurons];
				
		for(int i=0; i < pattern.length; i++) {
			// set input pattern array for jaNet
			validationPattern[i][0] = pattern[i].getX();
			validationPattern[i][1] = pattern[i].getY();
		}
		
		return validationPattern;
	}
	
	
	
	public void  performTraining()  throws RemoteException{
		
		if( _inputPattern == null || _outputPattern == null ) {
			System.out.println("Error can't train net without any input/output patterns!");
			return;
		}
		double nError = Double.MAX_VALUE;
		int nCycles=0;
		
		//System.out.println("Starting net training...");
		
		try {			
			for( ; 
			     nCycles < _nMaxTrainingCycles  && nError > _nMinimumError; 
			     nCycles++) {
			     	
				 // chose at random an input pattern
				 
				 int p = (int)(Math.random()*((double)_inputPattern.length));			     	
				 				 
				 // learn the input pattern in order to reproduce
				 // corresponding target		
				 /*System.out.println(" training pattern #" + p);
				 System.out.println("input: " + _inputPattern[p][0] + "," + _inputPattern[p][1] );
				 System.out.println("output: " + _outputPattern[p][0] );
				 */
				 try {
				 _bpn.learn( _inputPattern[p], _outputPattern[p]);
				
				 // compute global error making the sum of errors of all patterns			
			     nError = getCurrentError();	
			     }catch(NullPointerException e1) {
			     	System.out.println("" + e1.toString() );
			     }
				
			}
	   } catch(BPNException e) {
			System.out.println("Error while training net: " + e.toString());
	   }
	   
	   //System.out.println("Finished training the net: ");
	   System.out.println("Neterror: " + nError + " after " + nCycles + " training cycles" );	   
	}
	
	/**
	 * Sets the maximum training cycles.
	 * If given value is 0, no training limit is set
	 */
	public void setMaxTrainingCycles(int nMaxCycles) throws RemoteException {
		_nMaxTrainingCycles = nMaxCycles;
	}

	/**
	 * Sets the minimum error value
	 */
	public void setMinimumError(double nMinError)  throws RemoteException{
		_nMinimumError = nMinError;
	}

	/**
	 * Sets the learning rate of the net
	 */
	public void setLearningRate(double nLearnRate)  throws RemoteException{
		_nLearningRate = nLearnRate;
		_bpn.setLearningRate( nLearnRate);		
	}
	
	/**
	 * Sets the momentum of the NN
	 */
	public void setMomentum(double nMomentum)  throws RemoteException{
		_nMomentum = nMomentum;
		_bpn.setMomentum(nMomentum);
	}
	
	/**
	 * returns the errorvalue of the trained net
	 */
	public double getError()  throws RemoteException{
		return _nError;
	}
}