package edu.cosy.netgen.ann;

/**
 * Tangens hyperbolicus activation function.
 */
public class ActFunctionTanH extends ActFunction {
    
    /**
     * Create a tangens hyperbolicus activation function.
     */
    public ActFunctionTanH() {
        super();
    }

    /**
     * Compute the new value with the tangens hyperbolicus.
     * out = ( e^in - e^-in ) / ( e^in + e^-in )
     * <P>
     * @param value value to compute
     * @return new calculated value
     */
    public double actFunction( double value ) {
        return ( Math.exp( value ) - Math.exp( -value ) ) /
                 ( Math.exp( value ) + Math.exp( -value ) );
    }

    /**
     * Computes the derivation of the tangens hyperbolicus activation function.
     * out = 1.0 - in^2
     * <P> 
     * @param value the value to compute
     * @return new calculated value
     */
    public double actFunctionDerivation( double value ) {
        return 1.0 - sqr( value );
    }
}
