package edu.cosy.netgen.ann;

public class FitnessFunctionClassification extends FitnessFunction {
    
    /**
     * Create a new classification fitness function
     */
    public FitnessFunctionClassification() {
        super();
    }
        
    /**
     * Creates a new fitness function instance
     * @param patterns patterns
     * @param ffn net structure
     */
    public FitnessFunctionClassification( PatternSet validSet, FeedForwardNet ffn ) {
        super();
        init( validSet, ffn );
    }

    /**
     * Calculates fitness of the net, a classification function
     * @return fitness of the net (0=best fitness, 1=worst fitness)
     */
    public double getFitness() {
        
        //sum over all neurons, all patterns
        double sum = 0.0;
        
        //for all patterns
        for ( int j = 0; j < validationSet.getSize(); j++ ) {
            
                //which class, every output unit represents a class, pattern
                int indexPat = 0;
                
                //which class, every output unit represents a class, net output
                int indexOut = 0;
                
                //the biggest activation of an output unit
                int maxValue = 0;
                
                //search all output neurons
                for ( int k = 0; k < ffn.getOut(); k++ ) {
                    
                    //a 1 determines the class
                    if ( validationSet.getOutPatternArray()[ j ][ k ] == 1 )
                        indexPat = k;
                }
                
                //forward step for pattern j
                fpVali.initForwardCycle( j );
                fpVali.forwardCycle();
                
                //search for the biggest activation
                for ( int i = 0; i < ffn.getOut(); i++ ) {
                    if ( fpVali.outputvek[ i + 1 + ffn.getIn() + ffn.getHidden() ] > maxValue )
                        indexOut = i;
                }
                
                //if the pattern is correct classified
                if ( indexPat == indexOut )
                    sum++;
        }
        
        //1: no pattern is correct classified 0: all patterns are correct classified        
        if ( validationSet.getSize() == 0 )
            return 1.0;
        else
            return 1 - ( sum / validationSet.getSize() );
    }
}
