package edu.cosy.netgen.ann;
 
import java.util.*;

public abstract class NeuralNet implements java.io.Serializable {
    
    /**
     * Network type for Feed-Forward networks
     */
    public final static int FEEDFORWARD = 101;
    
    /**
     * Network type for Recurrent networks
     */
    public final static int RECURRENT = 102;

    /**
     * Number of in neurons
     */
    protected int in;

    /**
     * Number of out neurons
     */
    protected int out;

    /**
     * Number of hidden neurons
     */
    protected int hidden;

    /**
     * Contains the connections of the net
     */
    protected ConnectionMatrix conMatrix;

    /**
     * Contains the weights for the connections
     */
    protected WeightMatrix weightMatrix;

    /**
     * Act. func. used for each neuron
     */
    protected ActFunction[] actFuncNeuron;
    
    protected boolean markersModus = false;
    
    /**
     * Get the description of the network
     * @return description
     */
    public abstract String getDescription();
    
    protected void setIn( int value ) {
        in = value;
    }
    
    /**
     * Get number of input neurons
     * @return in neurons
     */
    protected int getIn() {
        return in;
    }

    protected void setHidden( int value ) {
        hidden = value;
    }
    
    /**
     * Get number of hidden neurons
     * @return hidden neurons
     */
    protected int getHidden() {
        return hidden;
    }

    protected void setOut( int value ) {
        out = value;
    }
    
    /**
     * Get number of output neurons
     * @return out neurons
     */
    protected int getOut() {
        return out;
    }

    /**
     * Get connection matrix
     * @return connection matrix
     */
    protected ConnectionMatrix getConMatrix() {
        return conMatrix;
    }

    protected void setConMatrix( ConnectionMatrix value ) {
        conMatrix = value;
    }
    
    protected void setMarkersModus( boolean value ) {
        markersModus = true;
    }
    
    protected boolean getMarkersModus() {
        return markersModus;
    }
    
    /**
     * Get weight matrix
     * @return weight matrix
     */
    protected WeightMatrix getWeightMatrix() {
        return weightMatrix;
    }

    /**
     * Set weight matrix
     * @param weightMatrix new weight matrix
     */
    protected void setWeightMatrix( WeightMatrix value ) {
        this.weightMatrix = value;
    }
    
    protected void setActFunctions( ActFunction[] values ) {
        actFuncNeuron = values;
    }
    
    /**
     * Get act. func. for each neuron
     * @return activation functions
     */
    protected ActFunction[] getActFunctions() {
        return actFuncNeuron;
    }
}
