package edu.cosy.netgen.ann;

/**
 * Class Pattern represents one pattern.
 */
public class Pattern implements java.io.Serializable {
    
    // array for the values
    private double p[];

    /**
     * Create a new pattern from a existing one.
     * <P>
     * @param tmp pattern to clone
     */
    public Pattern( Pattern tmp ) {
        this.p = new double[ tmp.getSize() ];
        System.arraycopy( tmp.getValues(), 0, p, 0, p.length );
    }

    /**
     * Create a new pattern.
     * <P>
     * @param size size of the pattern
     */
    public Pattern( int size ) {
        this.p = new double[ size ];
    }

    /**
     * Create a new pattern.
     * <P>
     * @param val patterns values
     */
    public Pattern( double[] val ) {
        this.p = new double[ val.length ];
        System.arraycopy( val, 0, p, 0, p.length );
    }
    
    /**
     * Get the size of the pattern.
     * <P>
     * @return size
     */
    public int getSize() {
        return p.length;
    }

    /**
     * Get a value of the pattern.
     * <P>
     * @param  i index of the value
     * @return value
     */
    public double getValue( int i ) {
        return p[ i ];
    }

    /**
     * Get pattern values
     * <P>
     * @return values
     */
    public double[] getValues() {
        return p;
    }
    
    /**
     * Set a value of the pattern.
     * <P>
     * @param i index of the value
     * @param d value to set
     */
    public void setValue( int i, double d ) {
        p[ i ] = d;
    }

    /**
     * Get a printable string that represents the pattern.
     * <P>
     * @return printable string
     */
    public String toString() {
        String tmp = "[";
        for ( int i = 0; i < p.length; i++ ) {
            tmp += p[ i ];
            if ( i < p.length - 1 )
                tmp += ", ";
        }
        return tmp + "]";
    }
}
