package edu.cosy.netgen.ann;

/**
 * Structure for a parameter of the training functions.
 */
public class TrainingParam implements java.io.Serializable {
    
    /**
     * Class of the parameter value.
     */
    protected Class type;
    
    /**
     * Description of the parameter.
     */
    protected String desc;
    
    /**
     * Value of the parameter.
     */
    protected Object val;
    
    /**
     * Minimum for paramter value.
     */
    protected Object minVal;
    
    /**
     * Maximum for parameter value.
     */
    protected Object maxVal;
    
    /**
     * Create a new instance with a selected type for value.
     * <P>
     * @param t class type for value
     */
    public TrainingParam( Class t ) {
        super();   
        this.type = t;
    }
    
    /**
     * Get class type of the value.
     * <P>
     * @return class of value
     */
    public Class getType() {
        return type;
    }
    
    /**
     * Set the parameter description.
     * <P>
     * @param value description
     */
    public void setDescription( String value ) {
        desc = value;
    }
    
    /**
     * Get the paramter description.
     * <P>
     * @return description
     */
    public String getDescription() {
        return desc;
    }
    
    /**
     * Set the value of the parameter. 
     * Throws java.lang.IllegalArgumentException if value has the wrong class type.
     * <P>
     * @param value new parameter value
     */
    public void setValue( Object value ) throws java.lang.IllegalArgumentException {
        if ( value != null && type.isInstance( value ) )
            val = value;
        else
            throw new IllegalArgumentException();
    }
    
    /**
     * Get the value of the paramter.
     * <P>
     * @return parameter value
     */
    public Object getValue() {
        return val;
    }
    
    /**
     * Set the minimum value of the parameter. 
     * Throws java.lang.IllegalArgumentException if value has the wrong class type.
     * <P>
     * @param value new minimum parameter value
     */
    public void setMinValue( Object value ) throws java.lang.IllegalArgumentException {
        if ( type.isInstance( value ) )
            minVal = value;
        else
            throw new IllegalArgumentException();
    }
    
    /**
     * Get the minimum value of the paramter.
     * <P>
     * @return minimum parameter value
     */
    public Object getMinValue() {
        return minVal;
    }
    
    /**
     * Set the maximum value of the parameter. 
     * Throws java.lang.IllegalArgumentException if value has the wrong class type.
     * <P>
     * @param value new maximum parameter value
     */
    public void setMaxValue( Object value ) throws java.lang.IllegalArgumentException {
        if ( type.isInstance( value ) )
            maxVal = value;
        else
            throw new IllegalArgumentException();
    }
    
    /**
     * Get the maximum value of the paramter.
     * <P>
     * @return maximum parameter value
     */
    public Object getMaxValue() {
        return maxVal;
    }
     
            
    /**
     * Set all values of the parameter. 
     * Throws java.lang.IllegalArgumentException if values have the wrong class type.
     * <P>
     * @param value new parameter value
     * @param value new minimum parameter value
     * @param value new maximum parameter value
     */
    public void setValues( Object value, Object min, Object max ) throws java.lang.IllegalArgumentException {
        setValue( value );
        setMinValue( min );
        setMaxValue( max );
    }

    /**
     * Get the values of the parameter, value, minimum and maximum.
     * <P>
     * @return parameter values
     */
    public Object[] getValues() {
        return new Object[]{ val, minVal, maxVal };
    }
}
