package edu.cosy.netgen.ann.ui;
 
import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.border.EtchedBorder;

import edu.cosy.netgen.ann.*;

public class ActivationWizardPage extends WizardPage {
    private JComboBox cbInput;
    private JComboBox cbHidden;
    private JComboBox cbOutput;
    private String[] functions;
    private int in, hidden, out;
    
    public ActivationWizardPage( String t ) {
        super( t );
    }
    
    public void setBean( ANN bean ) {
        super.setBean( bean );
        if ( bean != null ) {
            String[] tmp = bean.getActivationFunctionNames();
            for ( int i = 0; i < tmp.length; i++ ) {
                cbInput.addItem( tmp[ i ] );
                cbHidden.addItem( tmp[ i ] );
                cbOutput.addItem( tmp[ i ] );
            }
        }
    }

    public void initialize() {
        if ( values != null ) {
            in = ( ( Integer ) values.get( "in" ) ).intValue();
            hidden = ( ( Integer ) values.get( "hidden" ) ).intValue();
            out = ( ( Integer ) values.get( "out" ) ).intValue();
            functions = ( String[] ) values.get( "actFunctions" );
            if ( in > 0 && functions.length > in )
                cbInput.setSelectedItem( functions[ 1 ] );
            if ( hidden > 0 && functions.length > in + hidden )
                cbHidden.setSelectedItem( functions[ in + 1 ] );
            if ( out > 0 && functions.length > in + hidden + out )
                cbOutput.setSelectedItem( functions[ in + hidden + 1 ] );
        }
    }
    
    public void check() throws ANNBeanWizardException {
        boolean isError = true;
        functions = new String[ 1 + in + hidden + out ];
        String tmp = ( String ) cbInput.getSelectedItem();
        for ( int i = 1; i <= in; i++ )
            functions[ i ] = tmp;
        tmp = ( String ) cbHidden.getSelectedItem();
        for ( int i = in + 1; i <= in + hidden; i++ )
            functions[ i ] = tmp;
        tmp = ( String ) cbOutput.getSelectedItem();
        for ( int i = 1 + in + hidden; i < functions.length; i++ )
            functions[ i ] = tmp;
        values.remove( "actFunctions" );
        values.put( "actFunctions", functions );
        isError = false;
        if ( isError )
            throw new ANNBeanWizardException();
    }
        
    protected JPanel createValuePanel() {
        JPanel panel = new JPanel();
        panel.setBorder( new TitledBorder( new EtchedBorder(), " Layers " ) );

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        panel.setLayout( gridbag );
        c.anchor = GridBagConstraints.WEST;  

        JLabel label = new JLabel( "Input:" );
        c.insets = new Insets( 5, 10, 0, 0);
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 0;
        gridbag.setConstraints( label , c );
        panel.add( label );   
        
        cbInput = new JComboBox();
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 0;
        gridbag.setConstraints( cbInput , c );
        panel.add( cbInput );   
        
        label = new JLabel( "Hidden:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 1;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        cbHidden = new JComboBox();
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 1;
        gridbag.setConstraints( cbHidden , c );
        panel.add( cbHidden );   

        label = new JLabel( "Output:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 2;
        gridbag.setConstraints( label , c );
        panel.add( label );   
        
        cbOutput = new JComboBox();
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 2;
        gridbag.setConstraints( cbOutput , c );
        panel.add( cbOutput );   

        return panel;
    }
    
    private void select( JComboBox cb, ActFunction f ) {
        if ( ActFunctionLinear.class.isInstance( f ) )
            cb.setSelectedIndex( 0 );
        else if ( ActFunctionSigmoid.class.isInstance( f ) )
            cb.setSelectedIndex( 1 );
        else if ( ActFunctionTanH.class.isInstance( f ) )
            cb.setSelectedIndex( 2 );
    }
    
    private ActFunction getFunction( JComboBox cb ) {
        switch ( cb.getSelectedIndex() ) {
            case 0 : return new ActFunctionLinear();
            case 1 : return new ActFunctionSigmoid();
            case 2 : return new ActFunctionTanH();
            default : return null;
        }
    }
}
