package edu.cosy.netgen.ann.ui;
 
import java.awt.*;
import java.util.Hashtable;

import javax.swing.*;

import edu.cosy.netgen.ann.*;

public class StructureWizardPage extends WizardPage {    
    private JTextField tfDescription;
    private JComboBox cbNetType;
    private JComboBox cbConStyle;
    private JTextField tfInput;
    private JTextField tfHidden;
    private JTextField tfOutput;

    public StructureWizardPage( String t ) {
        super( t );
    }
            
    public void initialize() {
        if ( values != null ) {
            tfDescription.setText( ( String ) values.get( "description" ) );
            switch ( ( ( Integer ) values.get( "netType" ) ).intValue() ) {
                case edu.cosy.netgen.ann.NeuralNet.FEEDFORWARD : {
                    cbNetType.setSelectedIndex( 0 );
                    break;
                }
            }
            switch ( ( ( Integer ) values.get( "conStyle" ) ).intValue() ) {
                case edu.cosy.netgen.ann.ConnectionMatrix.CREATE_NONE : {
                    cbConStyle.setSelectedIndex( 0 );
                    break;
                } 
                case edu.cosy.netgen.ann.ConnectionMatrix.CREATE_BIAS_ONLY : {
                    cbConStyle.setSelectedIndex( 1 );
                    break;
                } 
                case edu.cosy.netgen.ann.ConnectionMatrix.CREATE_THREE_LAYER : {
                    cbConStyle.setSelectedIndex( 2 );
                    break;
                } 
                case edu.cosy.netgen.ann.ConnectionMatrix.CREATE_FULL : {
                    cbConStyle.setSelectedIndex( 3 );
                    break;
                } 
                case edu.cosy.netgen.ann.ConnectionMatrix.CREATE_RANDOM : {
                    cbConStyle.setSelectedIndex( 4 );
                    break;
                } 
            }
            tfInput.setText( ( ( Integer ) values.get( "in" ) ).toString() );
            tfHidden.setText( ( ( Integer ) values.get( "hidden" ) ).toString() );
            tfOutput.setText( ( ( Integer ) values.get( "out" ) ).toString() );
        }
    }
    
    public void check() throws ANNBeanWizardException {
        values.remove( "description" );
        values.put( "description", tfDescription.getText() );
        
        Integer tmp = null;        
        switch ( cbNetType.getSelectedIndex() ) {
            case 0 : {
                tmp = new Integer( edu.cosy.netgen.ann.NeuralNet.FEEDFORWARD );
                break;
            }
        }
        values.remove( "netType" );
        values.put( "netType", tmp );
        
        switch ( cbConStyle.getSelectedIndex() ) {
            case 0 : {
                tmp = new Integer( edu.cosy.netgen.ann.ConnectionMatrix.CREATE_NONE );
                break;
            }
            case 1 : {
                tmp = new Integer( edu.cosy.netgen.ann.ConnectionMatrix.CREATE_BIAS_ONLY );
                break;
            }
            case 2 : {
                tmp = new Integer( edu.cosy.netgen.ann.ConnectionMatrix.CREATE_THREE_LAYER );
                break;
            }
            case 3 : {
                tmp = new Integer( edu.cosy.netgen.ann.ConnectionMatrix.CREATE_FULL );
                break;
            }
            case 4 : {
                tmp = new Integer( edu.cosy.netgen.ann.ConnectionMatrix.CREATE_RANDOM );
                break;
            }
        }
        values.remove( "conStyle" );
        values.put( "conStyle", tmp );

        try {
            tmp = new Integer( tfInput.getText() );
            values.remove( "in" );
            values.put( "in", tmp );
        } catch ( Exception e ) {
            throw new ANNBeanWizardException( "Illegal number of input neurons" );
        }
        try {
            tmp = new Integer( tfHidden.getText() );
            values.remove( "hidden" );
            values.put( "hidden", tmp );
        } catch ( Exception e ) {
            throw new ANNBeanWizardException( "Illegal number of hidden neurons" );
        }
        try {
            tmp = new Integer( tfOutput.getText() );
            values.remove( "out" );
            values.put( "out", tmp );
        } catch ( Exception e ) {
            throw new ANNBeanWizardException( "Illegal number of output neurons" );
        }
    }
    
    protected JPanel createValuePanel() {
        JPanel panel = new JPanel();
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        panel.setLayout( gridbag );
        c.anchor = GridBagConstraints.WEST;  

        JLabel label = new JLabel( "Description:" ); 
        c.insets = new Insets( 5, 10, 0, 0);
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 0;
        gridbag.setConstraints( label , c );
        panel.add( label );   
        
        tfDescription = new JTextField( 20 );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 0;
        gridbag.setConstraints( tfDescription , c );
        panel.add( tfDescription );   
        
        label = new JLabel( "Network Type:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 1;
        gridbag.setConstraints( label , c );
        panel.add( label );   
        
        cbNetType = new JComboBox( new String[]{ "Feed Forward" } );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 1;
        gridbag.setConstraints( cbNetType , c );
        panel.add( cbNetType );   
        
        label = new JLabel( "Connection Type:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 2;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        cbConStyle = new JComboBox( new String[]{ "None", "Bias Only", "Three Layer", "Full", "Random" } );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 2;
        gridbag.setConstraints( cbConStyle , c );
        panel.add( cbConStyle );   

        label = new JLabel( "Input Neurons:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 3;
        gridbag.setConstraints( label , c );
        panel.add( label );   
        
        tfInput = new JTextField( 5 );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 3;
        gridbag.setConstraints( tfInput , c );
        panel.add( tfInput );   

        label = new JLabel( "Hidden Neurons:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 4;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        tfHidden = new JTextField( 5 );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 4;
        gridbag.setConstraints( tfHidden , c );
        panel.add( tfHidden );   

        label = new JLabel( "Output Neurons:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 5;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        tfOutput = new JTextField( 5 );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 5;
        gridbag.setConstraints( tfOutput , c );
        panel.add( tfOutput ); 
        
        return panel;
    }
}
