package edu.cosy.netgen.ann.ui;
 
import java.awt.*;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.border.EtchedBorder;

import edu.cosy.netgen.ann.*;

public class TrainingParamsWizardPage extends WizardPage {
    private TrainingParam[] params;
    private GridLayout layout;
    private JComponent[] fields;
    private String lastFunction = null;
    
    public TrainingParamsWizardPage( String t ) {
        super( t );
    }
    
    public void initialize() {
        if ( values != null ) {
            params = ( TrainingParam[] ) values.get( "trainParams" );
            String tmp = "edu.cosy.netgen.ann." + ( String ) values.get( "trainFunction" );
            if ( !( params != null && tmp.equals( lastFunction ) ) ) {
                Training function = null;
                try { 
                    function = ( Training ) Class.forName( tmp ).newInstance();
                } catch ( Exception e ) {
                }
                if ( function != null ) {
                    params = function.getParams();
                    lastFunction = tmp;
                }
            }
            if ( params != null ) {
                valuePanel.removeAll();
                fields = new JComponent[ params.length ];
                valuePanel.setBorder( new TitledBorder( new EtchedBorder(), " " + ( String ) values.get( "trainFunction" ) + " " ) );                
                GridBagLayout gridbag = new GridBagLayout();
                GridBagConstraints c = new GridBagConstraints();
                valuePanel.setLayout( gridbag );
                c.anchor = GridBagConstraints.WEST;
                c.insets = new Insets( 5, 10, 0, 0);                
                JLabel label;
                for ( int i = 0; i < params.length; i++ ) {
                    label = new JLabel( params[ i ].getDescription() + ":" );
                    c.weightx = 0.0;
                    c.gridx = 0;
                    c.gridy = i;
                    gridbag.setConstraints( label , c );
                    valuePanel.add( label );
                    if ( params[ i ].getType() == Double.class ) {
                        fields[ i ] = new JTextField( params[ i ].getValue().toString(), 5 );
                        c.weightx = 0.5;
                        c.gridx = 1;
                        gridbag.setConstraints( fields[ i ] , c );
                        valuePanel.add( fields[ i ] );
                    } else if ( params[ i ].getType() == Boolean.class ) {
                        fields[ i ] = new JCheckBox( "", ( ( Boolean ) params[ i ].getValue() ).booleanValue() );
                        c.weightx = 0.5;
                        c.gridx = 1;
                        gridbag.setConstraints( fields[ i ] , c );
                        valuePanel.add( fields[ i ] );
                    }
                }
            }
        }   
    }
    
    public void check() throws ANNBeanWizardException {
        if ( params != null ) {
            for ( int i = 0; i < params.length; i++ ) {
                if ( params[ i ].getType() == Double.class ) {
                    try {
                        params[ i ].setValue( new Double( ( ( JTextField ) fields[ i ] ).getText() ) );
                    } catch ( Exception e ) {
                        throw new ANNBeanWizardException( "Illegal value at " + params[ i ].getDescription() );
                    }
                } else if ( params[ i ].getType() == Boolean.class ) {
                    try {
                        params[ i ].setValue( new Boolean( ( ( JCheckBox ) fields[ i ] ).isSelected() ) );
                    } catch ( Exception e ) {
                        throw new ANNBeanWizardException( "Illegal value at " + params[ i ].getDescription() );
                    }
                }
            }
            values.remove( "trainParams" );
            values.put( "trainParams", params );
        }
    }
        
    protected JPanel createValuePanel() {
        JPanel panel = new JPanel();
        return panel;
    }
}
