package edu.cosy.netgen.client.design;
 
import java.util.*;
import java.rmi.*;
import java.io.*;
import java.net.InetAddress;

import edu.cosy.netgen.ann.*;
import edu.cosy.netgen.parser.*;
import edu.cosy.netgen.ga.ANNGenotypeBasic;
import edu.cosy.netgen.ga.ANNGenotypeTPSMarkers;
import edu.cosy.netgen.server.*;
import edu.cosy.netgen.server.service.*;
import edu.cosy.netgen.server.event.ServerEvent;
import edu.cosy.netgen.client.NetGenClient;

public class SimpleDesignClient implements NetGenClient {
    private DesignClientService clientService;
    private final static String CLIENT_PROPERTIES = "client.properties";
    private Properties clientProps;
    private long sessionID = -1;

    public SimpleDesignClient() {
        super();
        clientProps = new Properties();
        if ( readProperties( CLIENT_PROPERTIES ) && initialize() )
            work();
        else
            exit( 1 );
    }
    
    private boolean initialize() {
        boolean retVal = false;
        System.out.println( "Starting netGen Design Client ... " );
        if ( initializeRMI() && connect() ) {
            retVal = true;
        }
        return retVal;
    }
    
    private boolean initializeRMI() {
        boolean retVal = false;
        try {
            clientService = ( DesignClientService ) Naming.lookup( "rmi://" + clientProps.getProperty( "server.host" ) + ":" + clientProps.getProperty( "server.port" ) + "/netGenClientService" );
            if ( clientService != null )
                retVal = true;
        } catch ( Exception e ) {
            System.err.println( "Cannot connect to RMI server: " + e );
        }
        return retVal;
    }

    public void work() {
        try {
            AnnParser parser = new AnnXMLParser(); 
            ANN bean = parser.importNet( new File( "net.xml" ) ); 
            PatternSet trainingPatterns = parser.importPatterns( new File( "pat.xml" ) );
            PatternSet validationPatterns = parser.importPatterns( new File( "pat.xml" ) );
            PatternSet calculationPatterns = parser.importPatterns( new File( "pat.xml" ) );
                
            bean.setTrainingPatternSet( trainingPatterns );
            bean.setValidationPatternSet( validationPatterns );
            bean.setCalculationPatternSet( calculationPatterns );
            ANNGenotypeBasic genotype = new ANNGenotypeBasic( bean, true, false, false, false, false, false, 0.0, 1.0, new String[]{}, new String[]{}, new String[]{} );
            TaskCreationInfo info = new TaskCreationInfo( TaskCreationInfo.EVOLVE_ANN, bean, 10, 5, genotype );
            createNewTask( info );
            ANNGenotypeTPSMarkers genotype2 = new ANNGenotypeTPSMarkers( bean );
            TaskCreationInfo info2 = new TaskCreationInfo( TaskCreationInfo.EVOLVE_PATTERN, bean, 10, 5, genotype2 );
            createNewTask( info2 );
        } catch ( Exception e ) {
            System.out.println( "E:" + e);
        }        
    }
    
    public boolean connect() {
        boolean retVal = false;
        try {
            // set the session ID
            sessionID = clientService.connect( new DesignClientInfo( InetAddress.getLocalHost() ) );
            if ( sessionID != -1 )
                retVal = true;
        } catch ( Exception e ) {
        }
        return retVal;
    }

    /**
     * Disconnect from server
     * @return true, if successful
     */
    public boolean disconnect() {
        boolean retVal = false;
        try {
            if ( clientService != null && sessionID != -1 )
                retVal = clientService.disconnect( sessionID );
            if ( retVal ) {
                sessionID = -1;
                clientService = null;
            }
        } catch ( Exception e ) {
        }
        return retVal;
    }
    
    public long createNewTask( TaskCreationInfo info ) {
        long retVal = -1;
        try {
            retVal = clientService.createNewTask( sessionID, info );
        } catch ( Exception e ) {
        }
        return retVal;
    }

    private boolean readProperties( String file ) {
        boolean retVal = false;
        try {
            FileInputStream fin = new FileInputStream( new File( file ) );
            clientProps.load( fin );
            if ( fin != null )
                fin.close();
            retVal = true;
        } catch ( IOException ioe ) {            
            System.err.println( "Cannot read properties: " + ioe );
            retVal = false;
        }
        return retVal;
    }

    /**
     * Exit the client
     * @param state exit state
     */
    public void exit( int state ) {
        System.out.println( "Shut down netGen Design Client ... " );
        if ( state == 1)
            System.err.println( "Abnormal termination" );
        disconnect();
        System.exit( state );
    }

    public boolean processCommand( ClientControlCommand cmd )  {
        return false;
    }
    
    public void eventPerformed( ServerEvent e )  {
    }

    public static void main( String[] args ) {
        SimpleDesignClient main = null;
        switch ( args.length ) {
            default:
                main = new SimpleDesignClient();
        }
        if ( main == null ) {
            System.err.println( "Cannot start netGen Design Client!" );
            System.exit( 1 );
        }
        else {
            System.out.println( "netGen Design Client is working..." );
            System.out.print( "Type \"quit\" for exit ..." );
            try {
                byte[] command = new byte[ 4 ];
                System.in.read(command, 0, 4);
                if ( ( new String( command ) ).equals( "quit" ) )
                    main.exit( 0 );
            }
            catch (Exception e) {
            }
        }       
    }
}
