package edu.cosy.netgen.client.management;
 
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.rmi.Naming;
import java.rmi.RemoteException;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import edu.cosy.netgen.client.management.ui.ServerManagementTree;
import edu.cosy.netgen.client.management.ui.ServerManagementTreeControl;
import edu.cosy.netgen.misc.Util;
import edu.cosy.netgen.client.management.module.ModuleManager;
import edu.cosy.netgen.client.management.module.WorkingClientModule;
import edu.cosy.netgen.server.service.ServerControlService;
import edu.cosy.netgen.server.ManagementClientInfo;

public class ServerManager extends JFrame {
    public final static int UID_FILE_EXIT = 101;
    public final static int UID_SERVER_ADD = 201;
    
    private ServerManagementTreeControl treeControl;
    private JSplitPane splitPane;
    private ModuleManager moduleManager;
    private ActionAdapter actionAdapter;
    private ServerInfo connectedServer = null;
    
    class ActionAdapter implements ActionListener {
        public void actionPerformed( ActionEvent e ) {
            if ( e.getActionCommand().equals( Integer.toString( UID_FILE_EXIT ) ) )
                exit( 0 );    
        }
    }
    
    public ServerManager() {
        super( "netGen " + edu.cosy.netgen.misc.Constants.VERSION + " Server Manager" );
        moduleManager = new ModuleManager( this );
        moduleManager.addModule( new WorkingClientModule() );
        getContentPane().setLayout( new BorderLayout() );
        actionAdapter = new ActionAdapter();
        getContentPane().add( createMenu( actionAdapter ), BorderLayout.NORTH );
        getContentPane().add( createManagementPane(), BorderLayout.CENTER );
    }

    public ModuleManager getModuleManager() {
        return moduleManager;
    }
    
    public boolean connect( ServerInfo info ) {
        boolean retVal = false;
        if ( connectedServer != null ) {
            JOptionPane.showMessageDialog( this, "Only one connection is allowed!", "Error", JOptionPane.ERROR_MESSAGE );
            return retVal;
        }
        if ( connectedServer == null && info != null ) {
            try {            
                info.setControlService( ( ServerControlService ) Naming.lookup( "rmi://" + info.getAddress().getHostAddress() + ":" + info.getPort() + "/netGenControlService" ) );
                if ( info.getControlService() != null ) {
                    long id = info.getControlService().connect( new ManagementClientInfo( info.getAddress() ) );
                    if ( id != -1 ) {
                        info.setSessionID( id );
                        connectedServer = info;
                        retVal = true;
                    }
                }
            } catch ( Exception e ) {
                JOptionPane.showMessageDialog( this, "Cannot connect to netGen server!", "Error", JOptionPane.ERROR_MESSAGE );
                System.err.println( "Cannot connect to RMI server: " + e );
            }
        }
        return retVal;
    }
    
    public boolean disconnect( ServerInfo info ) {
        boolean retVal = false;
        if ( info != null && info.getControlService() != null ) {
            try {
                if ( info.getControlService().disconnect( info.getSessionID() ) ) {
                    info.setSessionID( -1 );
                    info.setControlService( ( ServerControlService ) null );
                    connectedServer = null;
                    retVal = true;
                }
            } catch ( RemoteException e ) {
                JOptionPane.showMessageDialog( this, "Cannot disconnect from netGen server!", "Error", JOptionPane.ERROR_MESSAGE );
            }
        }
        return retVal;
    }
    
    public void setModulePanel( JPanel p ) {
        if ( p == null )
            splitPane.setRightComponent( getDefaultModulePanel() );
        else
            splitPane.setRightComponent( p ); 
    }
    
    public ServerInfo getConnectedServer() {
        return connectedServer;
    }
    
    public void exit( int state ) {
        System.out.println( "Shut down netGen Management Client ... " );
        if ( state == 1)
            System.err.println( "Abnormal termination" );
        System.exit( state );
    }

    private JSplitPane createManagementPane() {
        splitPane = new JSplitPane();
        ServerManagementTree tree = new ServerManagementTree();
        treeControl = new ServerManagementTreeControl( tree, this );
        JScrollPane treeView = new JScrollPane( tree );        
        splitPane.setLeftComponent( treeView );
        splitPane.setRightComponent( getDefaultModulePanel() );
        return splitPane;
    }
    
    private JPanel getDefaultModulePanel() {
        JPanel retVal = new JPanel();
        retVal.add( new JLabel( "Default" ) );
        return retVal;
    }
    
    private JMenuBar createMenu( ActionListener al ) {
        JMenuBar menuBar = new JMenuBar();

        JMenu menu1 = new JMenu( "File", true );
        menu1.setMnemonic( 'F' );
        menu1.add( Util.makeImageMenuItem( "Exit", 'x', UID_FILE_EXIT, al, null, true ) );
        
        menuBar.add( menu1 );
        return menuBar;
    }
    
    public static void main( String[] args ) {
        final ServerManager mainFrame = new ServerManager();
        mainFrame.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                mainFrame.exit( 0 );
            }            
        } );
        mainFrame.show();
        mainFrame.pack();
    }
}
