package edu.cosy.netgen.client.management.module;
 
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.rmi.RemoteException;
import javax.swing.JPopupMenu;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JMenuItem;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;

import edu.cosy.netgen.misc.Util;
import edu.cosy.netgen.client.management.ServerInfo;
import edu.cosy.netgen.server.ClientControlCommand;
import edu.cosy.netgen.server.ClientInfo;
import edu.cosy.netgen.server.WorkingClientInfo;

public class WorkingClientModule extends ManagementModule {   
    class RowActionListener implements ActionListener {
        public void actionPerformed( ActionEvent e ) {
            WorkingClientInfo info = tableModel.getWorkingClientAtRow( clientTable.getSelectedRow() );
            if ( info != null && info.getControlService() != null ) {
                try {
                    int id = Integer.parseInt( e.getActionCommand() );
                    currentServer.getControlService().invokeWorkingClientCommand( ClientInfo.WORKING_CLIENT, clientCmds[ id ] );
                } catch ( Exception ex ) {
                    System.err.println( "Cannot invoke client command! " + ex );
                }
            }
        }
    }
    
    class TableMouseAdapter extends MouseAdapter {
        public void mouseClicked( MouseEvent e ) {
            if ( clientTable != null ) {
                int row = clientTable.getSelectedRow();
                if ( row > -1 && SwingUtilities.isRightMouseButton( e ) ) {
                    clientMenu.show( clientTable, e.getX(), e.getY() );                        
                }
            }
        }
    }
    
    protected ServerInfo currentServer = null;

    private final static String name = "Working Clients";
    private final static String description = "Working Client Manager 0.1";
    private WorkingClientTableModel tableModel;
    private JTable clientTable = null;
    private JPanel modulePanel;
    private JPopupMenu clientMenu = null;
    private ClientControlCommand[] clientCmds = null;
    
    public String getNodeName() {
        return name;
    }
    
    public String getDescription() {
        return description;
    }

    public JPopupMenu showModuleMenu( Component c, int x, int y ) {
        JPopupMenu menu = null;
        return menu;
    }
    
    public boolean load( ServerInfo info ) {
        if ( tableModel == null )
            tableModel = new WorkingClientTableModel();
        currentServer = info;
        if ( currentServer != null && currentServer.getControlService() != null && clientMenu == null ) {
            try {
                clientCmds = currentServer.getControlService().getWorkingClientCommands();
                if ( clientCmds != null ) {
                    clientMenu = new JPopupMenu();
                    RowActionListener al = new RowActionListener();
                    for ( int i = 0; i < clientCmds.length; i++ ) {
                        if ( !clientCmds[ i ].hasParameter() )  
                            clientMenu.add( Util.makeImageMenuItem( clientCmds[ i ].getCmd(), ( char ) -1, i, al, null, true ) );
                    }
                }
            } catch ( Exception e ) {
                System.err.println( "Cannot get WorkingClient commands! " + e );                
            }
        }
        return true;
    }
    
    public boolean unload() {
        return true;
    }
    
    public JPanel getModulePanel() {
        if ( modulePanel == null ) {
            modulePanel = new JPanel();
            modulePanel.setLayout( new BorderLayout() );
        
            modulePanel.add( new JLabel( description, JLabel.CENTER ), BorderLayout.NORTH );
        
            clientTable = new JTable( tableModel );
            clientTable.addMouseListener( new TableMouseAdapter() );
            clientTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
            
            JScrollPane scrollPane = new JScrollPane( clientTable );
            modulePanel.add( scrollPane, BorderLayout.CENTER );
        }
        return modulePanel;
    }
    
    public void refresh() {
        if ( tableModel != null && currentServer != null && currentServer.getControlService() != null ) {
            try {
                tableModel.setWorkingClientList( currentServer.getControlService().getWorkingClientList() );
            } catch ( Exception e ) {
                System.err.println( "Test:" + e );
            }
        }
    }
}
