package edu.cosy.netgen.client.management.ui;
 
import java.util.Enumeration;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;

import edu.cosy.netgen.client.management.ServerInfo;
import edu.cosy.netgen.client.management.ServerManager;
import edu.cosy.netgen.misc.Util;
import edu.cosy.netgen.client.management.module.ManagementModule;
import edu.cosy.netgen.client.management.ModuleInfo;

public class ServerManagementTreeControl {
    public final static int UID_SERVER_ADD = 101;
    public final static int UID_SERVER_CONNECT = 102;    
    public final static int UID_SERVER_DISCONNECT = 103;    
    public final static int UID_SERVER_REMOVE = 104;    

    protected ServerManager serverManager;
    protected ServerManagementTree tree;
    protected DefaultMutableTreeNode root;
    protected DefaultTreeModel model;
    private JPopupMenu rootMenu;
    private JPopupMenu currentServerMenu;
    private TreeActionAdapter actionAdapter;
    
    class TreeMouseAdapter extends MouseAdapter {
        public void mouseClicked( MouseEvent e ) {
            JTree tree = ( JTree ) e.getSource();
            DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) tree.getLastSelectedPathComponent(); 
            Object info = null;
            if ( node != null )
                info = node.getUserObject();
            if ( SwingUtilities.isRightMouseButton( e ) ) {
                if ( info != null ) {
                    if ( ServerInfo.class.isInstance( info ) ) {
                        currentServerMenu = ( ( ServerInfo ) info ).getPopupMenu();
                        currentServerMenu.show( tree, e.getX(), e.getY() );                        
                    } else if ( ModuleInfo.class.isInstance( info ) ) {
                        ManagementModule mod = ( ( ModuleInfo ) info ).getModule();
                        if ( mod != null )
                            mod.showModuleMenu( tree, e.getX(), e.getY() );
                    } else if ( node == root ) {
                        rootMenu.show( tree, e.getX(), e.getY() );
                    }
                }
            } else if ( SwingUtilities.isLeftMouseButton( e ) ) {
                if ( info != null ) {
                    if ( ModuleInfo.class.isInstance( info ) ) {
                        ManagementModule mod = ( ( ModuleInfo ) info ).getModule();                        
                        serverManager.setModulePanel( mod.getModulePanel() );
                        mod.refresh();
                    }
                }
            }
        }
    }

    class TreeActionAdapter implements ActionListener {
        public void actionPerformed( ActionEvent e ) {
            if ( e.getActionCommand().equals( Integer.toString( UID_SERVER_ADD ) ) ) {
                addServerInfo();
            } else if ( e.getActionCommand().equals( Integer.toString( UID_SERVER_REMOVE ) ) ) {
                removeServerInfo();
            } else if ( e.getActionCommand().equals( Integer.toString( UID_SERVER_CONNECT ) ) ) {
                connect();
            } else if ( e.getActionCommand().equals( Integer.toString( UID_SERVER_DISCONNECT ) ) ) {
                disconnect();
            }
        }
    }

    public ServerManagementTreeControl( ServerManagementTree tree, ServerManager manager ) {
        serverManager = manager;
        this.tree = tree;
        root = ( DefaultMutableTreeNode ) tree.getModel().getRoot();
        model = ( DefaultTreeModel ) tree.getModel();
        TreeMouseAdapter mouseAdapter = new TreeMouseAdapter();
        tree.addMouseListener( mouseAdapter );
        actionAdapter = new TreeActionAdapter();
        rootMenu = createRootPopup( actionAdapter );
    }
    
    public void addServerInfo() {
        ServerInfo info = AddServerDialog.showDialog();
        if ( info != null ) {
            info.setPopupMenu( createServerPopup( actionAdapter ) );
            addServerInfo( info );            
        }
    }
    
    public void addServerInfo( ServerInfo info ) {
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( info );
        model.insertNodeInto( childNode, root, root.getChildCount() );
        tree.scrollPathToVisible( new TreePath( childNode.getPath() ) );
    }
        
    public void removeServerInfo() {
        DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) tree.getLastSelectedPathComponent();                
        model.removeNodeFromParent( node );
    }

    public void removeServerInfo( ServerInfo info ) {
        Enumeration childs = root.children();
        DefaultMutableTreeNode tmp;
        while ( childs.hasMoreElements() ) {            
            tmp = ( DefaultMutableTreeNode ) childs.nextElement();
            if ( tmp.getUserObject() == info )
                model.removeNodeFromParent( tmp );
        }
    }

    private void connect() {
        DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) tree.getLastSelectedPathComponent();                
        if ( node != null && serverManager.connect( ( ServerInfo ) node.getUserObject() ) ) {
            ManagementModule[] mods = serverManager.getModuleManager().getModuleList();
            if ( mods != null && mods.length > 0 ) {
                for ( int i = 0; i < mods.length; i++ ) {
                    if ( serverManager.getModuleManager().loadModule( mods[ i ] ) ) {
                        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( new ModuleInfo( mods[ i ] ) );
                        model.insertNodeInto( childNode, node, node.getChildCount() );
                        tree.scrollPathToVisible( new TreePath( childNode.getPath() ) );
                    }
                }
            }
            ( ( JMenuItem ) currentServerMenu.getSubElements()[ 0 ] ).setEnabled( false );
            ( ( JMenuItem ) currentServerMenu.getSubElements()[ 1 ] ).setEnabled( true );
        }
    }
    
    private void disconnect() {
        DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) tree.getLastSelectedPathComponent();
        if ( node != null && serverManager.disconnect( ( ServerInfo ) node.getUserObject() ) ) {
            for ( int i = 0; i < node.getChildCount(); i++ ) {
                DefaultMutableTreeNode tmp = ( DefaultMutableTreeNode ) node.getChildAt( i );
                ModuleInfo info = ( ModuleInfo ) tmp.getUserObject();
                if ( serverManager.getModuleManager().loadModule( info.getModule() ) )
                    model.removeNodeFromParent( tmp );
            }
            if ( node.getChildCount() == 0 ) {
                ( ( JMenuItem ) currentServerMenu.getSubElements()[ 0 ] ).setEnabled( true );
                ( ( JMenuItem ) currentServerMenu.getSubElements()[ 1 ] ).setEnabled( false );
                serverManager.setModulePanel( null );
            }
        }
    }
    
    private JPopupMenu createRootPopup( ActionListener al ) {
        JPopupMenu retVal = new JPopupMenu();
        retVal.add( Util.makeImageMenuItem( "Add ...", 'A', UID_SERVER_ADD, al, null, true ) );
        return retVal;
    }
    
    private JPopupMenu createServerPopup( ActionListener al ) {
        JPopupMenu retVal = new JPopupMenu();
        retVal.add( Util.makeImageMenuItem( "Connect", 'C', UID_SERVER_CONNECT, al, null, true ) );
        retVal.add( Util.makeImageMenuItem( "Disconnect", 'D', UID_SERVER_DISCONNECT, al, null, false ) );
        retVal.add( Util.makeImageMenuItem( "Remove", 'R', UID_SERVER_REMOVE, al, null, true ) );
        return retVal;
    }
}
