package edu.cosy.netgen.client.worker.ui;
 
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JProgressBar;
import javax.swing.BoxLayout;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.cosy.netgen.client.worker.WorkingClient;
import edu.cosy.netgen.misc.PropertyHelper;
import edu.cosy.netgen.misc.SwingWorker;
import edu.cosy.netgen.server.WorkingClientInfo;
import edu.cosy.netgen.client.worker.event.FitnessWorkerEvent;
import edu.cosy.netgen.client.worker.event.FitnessWorkerListener;
import edu.cosy.netgen.client.worker.FitnessWorkerThread;
import edu.cosy.netgen.server.TaskCreationInfo;

public class WorkingClientMainPanel extends JPanel {
    private ControlPanelActionAdapter actionListener = null;
    private FitnessWorkerAdapter workerListener = null;
    private WorkingClient client = null;
    private PropertyHelper clientProperties = null;
    private JButton btConnect, btDisconnect, btExit;
    private JTextField tfAddress, tfPort, tfClientAddress, tfClientPort;
    private JTabbedPane tabPane;
    private JLabel lbSessionID, lbStatus, lbIndividual, lbTaskType;
    private JProgressBar pbTraining;
    private boolean isConnected = false;
    
    class ControlPanelActionAdapter implements ActionListener {
        public void actionPerformed( ActionEvent e ) {
            if ( e.getActionCommand().equals( "exit" ) ) {
                changeProperties();
                client.exit( 0 );
            } else if ( e.getActionCommand().equals( "disconnect" ) ) { 
                final SwingWorker worker = new SwingWorker() {
                    private boolean success = false;
                    
                    public Object construct() {
                        success = client.disconnect();
                        return null;
                    }
                      
                    public void finished() {
                        if ( success ) {
                            tabPane.setSelectedComponent( tabPane.getComponent( 0 ) );
                            tfAddress.setEnabled( true );
                            tfPort.setEnabled( true );
                            tfClientPort.setEnabled( true );
                            btConnect.setEnabled( true );
                            btDisconnect.setEnabled( false );
                            lbSessionID.setText( "-1" );
                            pbTraining.setValue( 0 );
                            resetStatus();
                        } else
                            JOptionPane.showMessageDialog( null, "Cannot disconnect from netGen server!", "Error", JOptionPane.ERROR_MESSAGE ); 
                    }
                };
                worker.start();    
            } else if ( e.getActionCommand().equals( "connect" ) ) {
                final SwingWorker worker = new SwingWorker() {
                    private boolean success = false;
                    
                    public Object construct() {
                        success = client.connect();
                        return null;
                    }
                      
                    public void finished() {
                        if ( success ) {
                            tabPane.setSelectedComponent( tabPane.getComponent( 1 ) );
                            tfAddress.setEnabled( false );
                            tfPort.setEnabled( false );
                            tfClientPort.setEnabled( false );
                            btConnect.setEnabled( false );
                            btDisconnect.setEnabled( true );
                            lbSessionID.setText( Long.toString( client.getClientInfo().getSessionID() ) );
                        } else
                            JOptionPane.showMessageDialog( null, "Cannot connect to netGen server!", "Error", JOptionPane.ERROR_MESSAGE ); 
                    }
                };
                worker.start();    
            }
        }
    }
    
    class FitnessWorkerAdapter implements FitnessWorkerListener {
        public void statusChanged( final FitnessWorkerEvent e ) {
            if ( e != null ) {
                switch ( e.getStatus() ) {
                    case FitnessWorkerThread.WAITING: {
                        lbStatus.setText( "waiting ..." );
                        pbTraining.setValue( 0 );                        
                        break;
                    }
                    case FitnessWorkerThread.WORKING: {
                        lbStatus.setText( "working ..." );
                        break;
                    }
                    default: lbStatus.setText( "unknown" );
                }
                lbIndividual.setText( Integer.toString( e.getIndividualIndex() ) );
                switch ( e.getTaskType() ) {
                    case TaskCreationInfo.EVOLVE_ANN: {
                        lbTaskType.setText( "ANN Structure Evolving" );
                        break;
                    }
                    case TaskCreationInfo.EVOLVE_PATTERN: {
                        lbTaskType.setText( "ANN Pattern Evolving" );
                        break;
                    }                   
                    default: lbTaskType.setText( "Unknown Task" );                    
                }
            }
        }

        public void progressMade( final FitnessWorkerEvent e ) {
            if ( e != null )
                pbTraining.setValue( e.getPercent() );   
        }
    }
        
    public WorkingClientMainPanel( WorkingClient c ) {
        super();
        actionListener = new ControlPanelActionAdapter();
        workerListener = new FitnessWorkerAdapter();
        client = c;
        setLayout( new BorderLayout() );
        tabPane = new JTabbedPane();
        tabPane.add( "Properties", createPropertyPanel() );
        tabPane.add( "Status", createStatusPanel() );
        add( tabPane, BorderLayout.CENTER );
        add( createControlPanel(), BorderLayout.SOUTH );
        initialize();
    }
    
    public void initialize() {
        try {
            tfClientAddress.setText( InetAddress.getLocalHost().getHostAddress() );
            if ( ( clientProperties = client.getProperties() ) != null ) {
                tfAddress.setText( clientProperties.getProperty( "server.host" ) );
                tfPort.setText( clientProperties.getProperty( "server.port" ) );
                tfClientPort.setText( clientProperties.getProperty( "client.port" ) );
            }
        } catch ( UnknownHostException e ) {
        }
    }
        
    public FitnessWorkerListener getFitnessWorkerListener() {
        return workerListener;
    }
    
    public void resetStatus() {
        lbStatus.setText( null );
        lbIndividual.setText( null );
        lbTaskType.setText( null );
    }
    
    public void changeProperties() {
        if ( !tfAddress.getText().equals( clientProperties.getProperty( "server.host" ) ) )
            clientProperties.setProperty( "server.host", tfAddress.getText() );
        if ( !tfPort.getText().equals( clientProperties.getProperty( "server.port" ) ) )
            clientProperties.setProperty( "server.port", tfPort.getText() );
        if ( !tfClientPort.getText().equals( clientProperties.getProperty( "client.port" ) ) )
            clientProperties.setProperty( "client.port", tfClientPort.getText() );
    }
    
    private JPanel createPropertyPanel() {
        JPanel panel = new JPanel();
        panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS ) );
        
        JPanel sp = new JPanel();
        sp.setBorder( new TitledBorder( new EtchedBorder(), " netGen Server " ) );
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        sp.setLayout( gridbag );
        c.anchor = GridBagConstraints.WEST;        
        
        JLabel label = new JLabel( "Host Address:" );
        c.insets = new Insets( 5, 10, 0, 0);
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 0;
        gridbag.setConstraints( label , c );
        sp.add( label );   
        
        tfAddress = new JTextField( 10 );        
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 0;
        gridbag.setConstraints( tfAddress , c );
        sp.add( tfAddress );           
        
        label = new JLabel( "Host Port:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 1;
        gridbag.setConstraints( label , c );
        sp.add( label );   
        
        tfPort = new JTextField( 4 );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 1;
        gridbag.setConstraints( tfPort , c );
        sp.add( tfPort );     
        
        JPanel cp = new JPanel();
        cp.setBorder( new TitledBorder( new EtchedBorder(), " netGen Client " ) );
        
        cp.setLayout( gridbag );
        c.anchor = GridBagConstraints.WEST;        
        
        label = new JLabel( "Local Address:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 0;
        gridbag.setConstraints( label , c );
        cp.add( label );   
        
        tfClientAddress = new JTextField( 10 );
        tfClientAddress.setEnabled( false );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 0;
        gridbag.setConstraints( tfClientAddress , c );
        cp.add( tfClientAddress );           
        
        label = new JLabel( "Local Port:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 1;
        gridbag.setConstraints( label , c );
        cp.add( label );   
        
        tfClientPort = new JTextField( 4 );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 1;
        gridbag.setConstraints( tfClientPort , c );
        cp.add( tfClientPort );     

        panel.add( sp );
        panel.add( cp );
        return panel;
    }
    
    private JPanel createStatusPanel() {
        JPanel panel = new JPanel();
        panel.setBorder( new TitledBorder( new EtchedBorder(), " Runtime Information " ) );
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        panel.setLayout( gridbag );
        c.anchor = GridBagConstraints.WEST;  
        
        JLabel label = new JLabel( "Session ID:" );
        c.insets = new Insets( 5, 10, 0, 0);
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 0;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        lbSessionID = new JLabel( "-1" );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 0;
        gridbag.setConstraints( lbSessionID , c );
        panel.add( lbSessionID );   
        
        label = new JLabel( "Status:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 1;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        lbStatus = new JLabel();
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 1;
        gridbag.setConstraints( lbStatus , c );
        panel.add( lbStatus );   

        label = new JLabel( "Current Individual:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 2;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        lbIndividual = new JLabel();
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 2;
        gridbag.setConstraints( lbIndividual , c );
        panel.add( lbIndividual );   

        label = new JLabel( "Type of task:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 3;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        lbTaskType = new JLabel();
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 3;
        gridbag.setConstraints( lbTaskType , c );
        panel.add( lbTaskType );   

        label = new JLabel( "Training progress:" );
        c.weightx = 0.0;
        c.gridx = 0;
        c.gridy = 4;
        gridbag.setConstraints( label , c );
        panel.add( label );   

        pbTraining = new JProgressBar( 0, 100 );
        pbTraining.setStringPainted( true );
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 4;
        gridbag.setConstraints( pbTraining , c );
        panel.add( pbTraining );   

        return panel;
    }
    
    private JPanel createControlPanel() {
        JPanel panel = new JPanel();
        
        btConnect = new JButton( "Connect" );
        btConnect.setActionCommand( "connect" );
        btConnect.addActionListener( actionListener );
        panel.add( btConnect );
        
        btDisconnect = new JButton( "Disconnect" );
        btDisconnect.setEnabled( false );
        btDisconnect.setActionCommand( "disconnect" );
        btDisconnect.addActionListener( actionListener );
        panel.add( btDisconnect );
        
        btExit = new JButton( "Exit" );
        btExit.setActionCommand( "exit" );
        btExit.addActionListener( actionListener );
        panel.add( btExit );
        return panel;
    }      
}
