package edu.cosy.netgen.ga;

import banda.genalg.Genotype;
import banda.genalg.BitChromosome;

import edu.cosy.netgen.ann.*;

/**
 * Genotype for encoding training pattern set markers.
 */
public class ANNGenotypeTPSMarkers extends Genotype {
    
    /**
     * Temporal copy of the original bean.
     */
    protected transient ANN bean = null;
    
    /**
     * Create a new genotype for encoding training pattern set markers.
     * <P>
     * @param bean ANN bean with a training pattern set
     */
    public ANNGenotypeTPSMarkers( ANN bean ) {        
        super( 1 );
        int length = 0;
        if ( bean.getTrainingPatternSet() != null )
            length = bean.getTrainingPatternSet().getSize();
        setChromosome( new BitChromosome( length ) );
    }

    /**
     * Copy constructor for this genotype.
     * <P>
     * @param genotype genotype to copy
     */
    public ANNGenotypeTPSMarkers( ANNGenotypeTPSMarkers genotype ) { 
        super( genotype );
        this.bean = genotype.bean;
    }
       
    /**
     * Get the phenotype of this genotype, a boolean array for the pattern set markers.
     * <P>
     * @return array of training pattern set markers
     */
    public Object getPhenotype() {
        Boolean[] retVal = null;
        BitChromosome chrom = ( BitChromosome ) getChromosome();
        if ( chrom.length() > 0 ) {
            retVal = new Boolean[ chrom.length() ];
            for ( int i = 0; i < retVal.length; i++ )
                retVal[ i ] = new Boolean( chrom.get( i ) );
        }
        return retVal;
    }
 
    /**
     * Set the master bean for fitness calculation.
     * <P>
     * @param bean master bean
     */   
    public void setBean( ANN bean ) {
        this.bean = bean;
    }
    
    /**
     * Get the fitness of the master bean with the new training set.
     * <P>
     * @return bean fitness
     */
    public double getRawFitness() {
        return getRawFitness( null );
    }
    
    /**
     * Get the fitness of the master bean with the new training set.
     * <P>
     * @param listener lsitener for training progress
     * @return bean fitness
     */
    public double getRawFitness( edu.cosy.netgen.ann.ProgressListener listener ) {
        double retVal = 0.0;
        if ( bean != null ) {
            PatternSet pst = bean.getTrainingPatternSet();
            Boolean[] pt = ( Boolean[] ) getPhenotype();
            boolean[] tmp = new boolean[ pt.length ];
            for ( int i = 0; i < tmp.length; i++ )
                tmp[ i ] = pt[ i ].booleanValue();
            if ( pst != null && tmp.length == pst.getSize() ) {
                pst.setMarkers( tmp );
                if ( listener != null )
                    bean.addProgressListener( listener );
                bean.getWeightMatrix().createRandomWeights( bean.getConMatrix() );
                bean.trainNet();
                retVal = 1 - bean.getFitness();
            }
        }
        return retVal;
    }
    
     /**
     * Function for cloning.
     * <P>
     * @return new clone object
     */
    public Object clone() {
        return new ANNGenotypeTPSMarkers( this );
    }
}
