package edu.cosy.netgen.misc;
 
import java.util.*;
import java.io.*;

public class PropertyHelper {
    private File propFile;
    private String header;
    private Properties properties;
    private boolean isChanged;
    
    public PropertyHelper( String fn, String header ) {
        propFile = new File( fn );
        this.header = header;
    }
    
    public boolean open() {
        boolean retVal = false;
        if ( propFile != null && propFile.exists() ) {
            retVal = readProperties();
            isChanged = false;
        }
        return retVal;
    }
    
    public void close() {
        if ( isChanged )
            writeProperties();    
    }

    public void addProperty( String pn, String val ) {        
        if ( properties != null && ( !properties.containsKey( pn ) ) ) {
            properties.setProperty( pn, val );
            isChanged = true;
        }
    }

    public boolean setProperty( String pn, String val ) {
        if ( properties != null && properties.containsKey( pn ) ) {
            properties.setProperty( pn, val );
            isChanged = true;
            return true;
        } else
            return false;
    }
    
    public String getProperty( String pn ) {
        String retVal = null;
        if ( properties != null && properties.containsKey( pn ) )
            retVal = properties.getProperty( pn ).trim();
        return retVal;
    }
    
    public void addBooleanProperty( String pn, boolean val ) {
        if ( properties != null && ( !properties.containsKey( pn ) ) ) {
            properties.setProperty( pn, ( new Boolean( val ) ).toString() );
            isChanged = true;
        }
    }
    
    public boolean setBooleanProperty( String pn, boolean val ) {
        if ( properties != null && properties.containsKey( pn ) ) {
            properties.setProperty( pn, ( new Boolean( val ) ).toString() );
            isChanged = true;
            return true;
        } else
            return false;
    }
    
    public boolean getBooleanProperty( String pn ) {
        return Boolean.valueOf( this.getProperty( pn ).toLowerCase() ).booleanValue();
    }

    public String[] getPropertyNames() {
        String[] retVal = null;
        if ( properties != null ) {
            retVal = new String[ properties.size() ];            
            Enumeration entries = properties.propertyNames();
            int i = 0;
            while ( entries.hasMoreElements() ) {
                retVal[ i ] = ( String ) entries.nextElement();             
                i++;
            }
        }
        return retVal;
    }

    public Properties getProperties() {
        return properties;
    }
    
    private boolean readProperties() {
        boolean retVal = false;
        try {
            FileInputStream fin = new FileInputStream( propFile );
            if ( fin != null ) {
                properties = new Properties();
                properties.load( fin );
                fin.close();
                retVal = true;
            }
        } catch ( IOException ioe ) {            
            System.err.println( "Cannot read properties: " + ioe );
            properties = null;
            retVal = false;
        }
        return retVal;
    }
        
    private void writeProperties() {
        if ( properties != null ) {
            try {
                FileOutputStream fout = new FileOutputStream( propFile );
                if ( fout != null )
                    properties.store( fout, header );    
            } catch ( IOException ioe ) {            
                System.err.println( "Cannot write properties: " + ioe );
            }
        }
    }
}
