package edu.cosy.netgen.parser;

import java.io.File;

import edu.cosy.netgen.ann.*;

public interface AnnParser {
    
    /**
     * Import net structure from file
     * @param file file of the structure
     * @return instance of the bean
     */  
    public ANN importNet( File file ) throws AnnParserException;
    
    /**
     * Export net structure class to file
     * @param file file name
     * @param bean bean instance
     * @return true, if success
     */
    public boolean exportNet( File file, ANN bean ) throws AnnParserException;
    
    /**
     * Import pattern set from file
     * @param file pattern set file
     * @return new pattern set
     */
    public PatternSet importPatterns( File file ) throws AnnParserException;
    
    /**
     * Export pattern set to file
     * @param file pattern set file
     * @param patterns pattern set
     * @return true, if success
     */
    public boolean exportPatterns( File file, PatternSet patterns ) throws AnnParserException;
    
    /**
     * Get the description of this parser
     * @return description
     */
    public String getDescription();
    
    /**
     * Get the valid net file extensions
     * @return valid extensions for the file
     */
    public String[] getNetFileExt();
    
    /**
     * Get the valid pattern extensions
     * @return valid extensions for the file
     */
    public String[] getPatternFileExt();
}
