package edu.cosy.netgen.server;

import java.util.Hashtable;
import java.util.Properties;
import java.beans.Beans;
import org.log4j.Category;

import edu.cosy.netgen.misc.PropertyHelper;

/**
 * Abstract class for netGen repositories.
 */
public abstract class Repository {
    static Category CAT = Category.getInstance( "Repository" );    
    
    /**
     * Property Helper for the Repository.
     */
    protected static PropertyHelper properties = null;
    
    /**
     * Property preamble.
     */
    protected final static String propKey = "repository";
            
    /**
     * Create a new repository. If this is not possible to create one, create the default repository.
     * <P>
     * @param props property helper
     * @param rep default repository to use
     * @return new constructed repository
     */
    public static Repository createRepository( PropertyHelper props, String rep ) {
        Repository retVal = null;
        if ( props != null ) {
            properties = props;
            String tmp = properties.getProperty( propKey );
            if ( tmp == null && rep != null ) {
                tmp = rep;
                properties.setProperty( "repository", tmp );
            }
            try {
                retVal = ( Repository ) Beans.instantiate( null, tmp );
            } catch ( Exception e ) {
                return null;
            }
            if ( !retVal.initRepository() )
                retVal = null;
        }
        return retVal;
    }
    
    /**
     * Store a task in the repository.
     * <P>
     * @param task task to store
     * @return true, if success
     */
    public abstract boolean storeTask( Task task );
    
    /**
     * Load a task from repository.
     * <P>
     * @param id task task to be loaded
     * @return task, null otherwise
     */
    public abstract Task loadTask( long id );   
    
    /**
     * Check if a task is stored in the repository.
     * <P>
     * @param id task ID to check
     * @return true, if its stored
     */
    public abstract boolean isTaskStored( long id );
 
    /**
     * List the IDs of all stored tasks.
     * <P>
     * @return array of task IDs
     */
    public abstract long[] listStoredTasks();
    
    /**
     * Delete a task from the repository.
     * <P>
     * @param id task ID of the deleting task
     * @return true, if deleting was a success
     */
    public abstract boolean deleteStoredTask( long id );
 
    /**
     * Store the list of working clients.
     * <P>
     * @param list working client list
     * @return true, if success
     */
    public abstract boolean storeWorkingClientList( WorkingClientInfo[] list );
    
    /**
     * Load the working client list.
     * <P>
     * @return array of working client information
     */
    public abstract WorkingClientInfo[] loadWorkingClientList();
        
    /**
     * Close the repository.
     */
    public abstract void closeRepository();

    /**
     * Initialize the repository.
     * <P>
     * @return true, if it can be initialized
     */
    protected abstract boolean initRepository();
}
