package edu.cosy.netgen.server;
 
import banda.genalg.*;

import edu.cosy.netgen.ann.ANN;
import edu.cosy.netgen.ann.PatternSet;

public class TaskCreationInfo implements java.io.Serializable {
    public final static int EVOLVE_ANN = 1;
    public final static int EVOLVE_PATTERN = 2;
        
    private ANN bean;
    private PatternSet trainingSet;
    private PatternSet validationSet;
    private int populationSize = 0;
    private int generations = 0;
    private Genotype genotype;  
    private Selector selector;
    private Alterer alterer;
    private Allocator allocator;
    private Scaler scaler;
    private double survivorFraction;
    private double offspringFraction;
    private boolean contractive;
      
    private int taskType = -1;
    
    public TaskCreationInfo( int type, ANN bean, int popSize, int gen, Genotype g ) {
        this( type, bean, popSize, gen, g, new TournamentSelector(), new BasicAlterer(), 
              new FixedAllocator(), new LinearScaler(), 0, 1, false );
    }

    public TaskCreationInfo( int type, ANN bean, int popSize, int gen, Genotype g,
                             Selector selector, Alterer alterer, Allocator allocator, 
                             Scaler scaler, double survivorFraction, double offspringFraction, boolean contractive ) {
        super();
        if ( popSize > -1 )
            populationSize = popSize;
        if ( gen > -1 )
            generations = gen;
        genotype = g;
        taskType = type;
        this.bean = bean;
        if ( bean != null ) {
            trainingSet = bean.getTrainingPatternSet();
            validationSet = bean.getValidationPatternSet();            
        }            
        this.selector = selector;
        this.alterer = alterer;
        this.allocator = allocator;
        this.scaler = scaler;
        this.survivorFraction = survivorFraction;
        this.offspringFraction = offspringFraction;
        this.contractive = contractive;
    }
                             
    public ANN getBean() {
        return bean;
    }
    
    public PatternSet[] getWorkingPatternSets() {
        PatternSet[] retVal = null;
        if ( trainingSet != null && validationSet != null ) {
            retVal = new PatternSet[ 2 ];
            retVal[ 0 ] = trainingSet;
            retVal[ 1 ] = validationSet;
        }
        return retVal;
    }
    
    public int getPopulationSize() {
        return populationSize;
    }
    
    public int getGenerations() {
        return generations;
    }
    
    public Genotype getGenotype() {
        return genotype;
    }
    
    public int getTaskType() {
        return taskType;
    }
    
    public GeneticAlgorithm createGeneticAlgorithm() {
        GeneticAlgorithm retVal = null;
        try {
            retVal = new GeneticAlgorithm( genotype, selector, alterer, allocator, scaler, survivorFraction, offspringFraction, contractive );
        } catch ( Exception e ) {
        }
        return retVal;        
    }
}
