package edu.cosy.netgen.server;
 
import java.util.Hashtable;

import banda.genalg.Population;
import banda.genalg.Genotype;
import banda.genalg.Evaluation;

public class TaskResult implements java.io.Serializable {
    private long taskID = -1;
    private double bestFitness = 0.0;
    private int bestIndex = -1;
    private Hashtable container;    
    
    public TaskResult( Task task ) {
        if ( task != null )
            taskID = task.getTaskID();
    }

    public void setTaskID( long id ) {
        taskID = id;
    }
    
    public long getTaskID() {
        return taskID;
    }

    public void addPopulation( int g, Population p ) {
        if ( p != null ) {
            Evaluation eval = new Evaluation( p );            
            Genotype tmpBest = eval.getBestGenotype();
            if ( container == null )
                container = new Hashtable();
            container.put( new Integer( g ), eval );
            double tmp;
            if ( ( tmp = eval.getBestGenotype().getFitness() ) >= bestFitness ) {
                bestFitness = tmp;
                bestIndex = g;
            }
        }
    }
        
    public Evaluation getEvaluation( int g ) {
        Evaluation retVal = null;
        Integer key = new Integer( g );
        if ( container != null && container.containsKey( key ) )
            retVal = ( Evaluation ) container.get( key );
        return retVal;
    }
    
    public Genotype getBestGenotype() {
        Genotype retVal = null;
        if ( bestIndex > -1 ) {
            Evaluation tmp = getEvaluation( bestIndex );
            if ( tmp != null )
                retVal = tmp.getBestGenotype();
        }
        return retVal;
    } 

    public int getNumberOfGenerations() {
        return container.size();
    }
    
    public String toString() {
        StringBuffer retVal = new StringBuffer( "Result of task " );
        retVal.append( taskID ).append( ":\n" );
        retVal.append( "Number of generations: " ).append( getNumberOfGenerations() ).append( "\n" );
        if ( getBestGenotype() != null ) {
            retVal.append( "Best genotype ( fitness ): " ).append( getBestGenotype() );
            retVal.append( " ( " ).append( getBestGenotype().getFitness() ).append( " )\n" );
        }
        return retVal.toString();
    }
}
