package edu.cosy.netgen.server.service;

import java.rmi.server.*;
import java.rmi.RemoteException;

import edu.cosy.netgen.server.*;

public class DesignClientServiceImpl extends UnicastRemoteObject implements DesignClientService {
    private ClientManager clientManager;
    
    // reference for NetGenServer
    private NetGenServer server;
        
    /**
     * Create new instance for the client service
     * @param server NetGenServer instance
     * @param max maximum number of clients
     */
    public DesignClientServiceImpl( NetGenServer server, ClientManager manager ) throws RemoteException {
        super();
        this.server = server;
        this.clientManager = manager;
    }
    
    /**
     * Connect a client and return session ID
     * @return session ID
     */
    public long connect( DesignClientInfo info ) {
        if ( clientManager != null )
            return clientManager.connect( info );
        else
            return -1;
    }
    
    /**
     * Disconnect a special client
     * @param sessionID client session ID
     * @return true, if successful
     */
    public boolean disconnect( long sessionID ) {
        if ( clientManager != null )
            return clientManager.disconnect( sessionID );
        else
            return false;
    }
        
    /**
     * Create a new task with a specific type on the server
     * @param sessionID client session ID
     * @param type task type
     * @param bean object for working
     * @return task ID
     */
    public long createNewTask( long sessionID, TaskCreationInfo info ) {
        return server.createTask( info );
    }
    
    /**
     * Get information about a special task
     * @param sessionID client session ID
     * @param taskID task ID
     * @return task information
     */
    public TaskRuntimeInfo getTaskInformation( long sessionID, long taskID ) {
        return server.getTaskInformation( taskID );
    }
    
    public TaskRuntimeInfo[] getTaskList( long sessionID ) {
        return server.getTaskList();
    }

    /**
     * Get the result of a special task and removes task on the server
     * @param sessionID client session ID
     * @param taskID task ID
     * @return result object
     */
    public Object getTaskResult( long sessionID, long taskID, boolean remove ) {
        return server.getTaskResult( taskID, remove );
    }

    public boolean removeTask( long sessionID, long taskID, boolean remove ) {
        return server.removeTask( taskID, remove );
    }
}
