package edu.cosy.netgen.server.service;

import java.rmi.server.*;
import java.rmi.RemoteException;

import edu.cosy.netgen.server.*;

public class ServerControlServiceImpl extends UnicastRemoteObject implements ServerControlService {

    // reference for NetGenServer
    private NetGenServer server;    
    private ClientManager clientManager;
    
    /**
     * Create a new instance of the service
     * @param server NetGenServer instance
     */
    public ServerControlServiceImpl( NetGenServer server, ClientManager manager ) throws RemoteException {
        super();
        this.server = server;
        clientManager = manager;
    }

    /**
     * Connect a client and return session ID
     * @return session ID
     */
    public long connect( ManagementClientInfo info ) {
        if ( clientManager != null )
            return clientManager.connect( info );
        else
            return -1;
    }
    
    /**
     * Disconnect a special client
     * @param sessionID client session ID
     * @return true, if successful
     */
    public boolean disconnect( long sessionID ) {
        if ( clientManager != null )
            return clientManager.disconnect( sessionID );
        else
            return false;
    }

    public boolean startServer() {
        if ( server != null && !server.isRunning() ) {
            server.start();
            return true;
        } else
            return false;
    }
    
    public boolean stopServer() {
        if ( server != null && server.isRunning() ) {
            server.stop();
            return true;
        } else
            return false;
    }
    
    public boolean addWorkingClient( WorkingClientInfo info ) {
        if ( clientManager != null && info != null )
            return clientManager.addWorkingClient( info );
        else
            return false;
    }
     
    public boolean removeWorkingClient( long id ) {
        if ( clientManager != null )
            return clientManager.removeWorkingClient( id );
        else
            return false;
    }
    
    public WorkingClientInfo[] getWorkingClientList() {
        if ( clientManager != null )
            return clientManager.getWorkingClientList( false );
        else
            return null;
    }    

    public DesignClientInfo[] getDesingClientList() throws RemoteException {
        if ( clientManager != null )
            return clientManager.getDesignClientList();
        else
            return null;
    }    

    public ClientControlCommand[] getWorkingClientCommands() throws RemoteException {
        if ( clientManager != null )
            return clientManager.getWorkingClientCommands();
        else
            return null;
    }
    
    public int invokeWorkingClientCommand( int type, ClientControlCommand cmd ) throws RemoteException {
        if ( clientManager != null )
            return clientManager.broadcastControlCommand( type, cmd );
        else
            return 0;
    }
}
