////////////////////////////////////////////////////////////////////////////////////  
//  
//  Copyright (C) 1996 L.  Patocchi & W.Gander
//
//  This program is free  software;  you can  redistribute  it and/or  modify it
//  under the terms of the GNU General  Public  License as published by the Free
//  Software  Foundation;  either  version 2 of the License, or (at your option)
//  any later version.
//
//  This program is  distributed in the hope that it will be useful, but WITHOUT
//  ANY  WARRANTY;  without  even the implied  warranty  of  MERCHANTABILITY  or
//  FITNESS FOR A PARTICULAR  PURPOSE.  See the GNU General  Public  License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software  Foundation, Inc., 675 Mass
//  Ave, Cambridge, MA 02139, USA.
//
//  Contacts:
//  
//    Project Supervisor
//      W.Hett       hew@info.isbiel.ch
//  
//    Authors
//      W.Gander     gandw@info.isbiel.ch
//      L.Patocchi   patol@info.isbiel.ch
//
//  Documentation can be found at:
//
//      http://www.isbiel.ch/Projects/janet/index.html
//
////////////////////////////////////////////////////////////////////////////////////
//
// File : jaNet.java
//
/*
	jaNet - A Neural Network Toolkit for Java
	-----------------------------------------

	Author :		W.Gander
	Modifications :	10.09.1996
	Filename :		jaNet.java

	Description :	jaNet Main class


	(c) 1996 Biel School of Engineering
 */
package jaNet;

import java.awt.*;
import java.io.*;
import java.util.*;

public class jaNet extends Frame implements Confirmable,Runnable{

	public static final	String	version 		= "1.0beta1";
	public static final	String	homePage 		= "http://www.isbiel.ch/~patol/janet";
	public static final	String	NOTHING 		= "NOTHING";
	public static final	int		QUIT_CONFIRM 			= 0;
	public static final	int		QUIT_SAVEALL_CONFIRM 	= 1;
	public static final	int		QUIT_ASK_CONFIRM 		= 2;
	public static final	int		PRJ_SAVE	 			= 3;

	// Declarations
	private	String			jaNetCFGFile 		= "jaNet.cfg";
	private	String			jaNetCFGPath 		= ".";
	private	String			jaNetHomePath 		= ".";
	private boolean			isAProjectOpen 		= false;
	
	private jaNetProject	currentProject		= null;
	
	private	FileDialog		openProjectFD		= null;
	private	FileDialog		createProjectFD		= null;	

	private MenuItem		menuNew;
	private MenuItem		menuOpen;
	private MenuItem		menuEdit;
	private MenuItem		menuClose;
	

	private	OptionsBox		jaNetOptionsBox;
	private	confirmDialog	jaNetConfirmDialog;
	private	ModuleList		jaNetModuleList;
	private	AboutBox		jaNetAboutBox;
	
	private	Vector			moduleTypeTable;
	private	ModuleTypes		modt				= null;
		
	// thread guard
	private Thread 			threadGuard 		= null;
	public String			todo				= NOTHING;

    public Menu 			moduleMenu;
    public Menu 			newModuleMenu;

//////////////////////////////////////////////////////////////////
//
//	MAIN method
//
//////////////////////////////////////////////////////////////////
    public static void main(String args[]) {
		jaNet myJaNet = new jaNet(args);
    }

//////////////////////////////////////////////////////////////////
//
//	jaNet creator and commandline attributes parser
//
//////////////////////////////////////////////////////////////////
	public jaNet(String	args[])	{
		super("jaNet");
		addNotify();
		
		Menu fileMenu;
		Menu projectMenu;
		Menu helpMenu;

		int i=0;
		// parse imput commands
		while(i<args.length){
			if(args[i].compareTo("-cfgpath")==0 && i< args.length-1){
				i++;
				jaNetCFGPath = args[i];
			}else
			if(args[i].compareTo("-cfgfile")==0 && i< args.length-1){
				i++;
				jaNetCFGFile = args[i];
			}else
			if(args[i].compareTo("-homepath")==0 && i< args.length-1){
				i++;
				jaNetHomePath = args[i];
			}
			i++;
		}

		moduleTypeTable	= new Vector();
		
		readConfigFile(jaNetCFGPath, jaNetCFGFile);
		
		MenuBar mb = new MenuBar();
		fileMenu = new Menu("File");
		fileMenu.add(new MenuItem("Options"));
		fileMenu.addSeparator();
		fileMenu.add(new MenuItem("Exit and Save All"));
		fileMenu.add(new MenuItem("Exit and Discard"));
		fileMenu.add(new MenuItem("Exit"));
		mb.add(fileMenu);
		projectMenu	= new Menu("Project");
		projectMenu.add((menuNew = new	MenuItem("New...")));
		projectMenu.add((menuOpen= new	MenuItem("Open...")));
		projectMenu.add((menuEdit= new	MenuItem("Edit...")));
		menuEdit.disable();
		projectMenu.add((menuClose= new	MenuItem("Close")));
		menuClose.disable();
		mb.add(projectMenu);
		moduleMenu = new Menu("Module");
		newModuleMenu =	new	Menu("New");

		// creates the new module menu on the fly
		for(int j=0; j< moduleTypeTable.size(); j++){
			newModuleMenu.add(new MenuItem(((ModuleTypes)moduleTypeTable.elementAt(j)).ModuleTypeName));
		}
	
		moduleMenu.add(newModuleMenu);
		moduleMenu.add(new MenuItem("Module list"));
		mb.add(moduleMenu);
		helpMenu = new Menu("Help");
		helpMenu.add(new MenuItem("Contents"));
		helpMenu.add(new MenuItem("Java API"));
		helpMenu.addSeparator();
		helpMenu.add(new MenuItem("About..."));
		mb.add(helpMenu);
		setMenuBar(mb);
		//Display the help menu	in a special reserved place.
		mb.setHelpMenu(helpMenu);

		setLayout(null);
		if(System.getProperty("file.separator").compareTo("/") == 0){
			resize(1280, insets().top +	insets().bottom);
		}else{
			resize(1024, insets().top +	insets().bottom+50);		
		}
		pack();
		this.show();
		setProjectInactive();
		// start Thread guard
		start();
	}

	public synchronized	void show()	{
		move(0,	0);
		super.show();
	}

	public void paint(Graphics g) {
		move(0, 0);
	}

	public void exit(){
		hide();											// hide the Frame
		writeConfigFile(jaNetCFGPath, jaNetCFGFile);	// write config
		dispose();										// tell windowing system to free resources
		System.exit(0);									// exit
	}
//////////////////////////////////////////////////////////////////
//
//	events handlers
//
//////////////////////////////////////////////////////////////////
	public boolean handleEvent(Event event)	{
		if(event.id == Event.WINDOW_MOVED){
			//move(0, 0);
		}else
		if(event.id ==	Event.WINDOW_DESTROY) {
			jaNetConfirmDialog = new confirmDialog(this, "Are You sure to quit jaNet ?", QUIT_CONFIRM);
			jaNetConfirmDialog.show();
			return true;
		}
		return super.handleEvent(event);
	}

    public boolean action(Event event, Object arg) {
		if (event.target instanceof MenuItem) {
		    String label = (String) arg;
		    if (label.equalsIgnoreCase("Options")) {
		        selectedOptions();
		        return true;
		    } else if (label.equalsIgnoreCase("Exit and Save All")) {
		        selectedExitSaveAll();
		        return true;
		    } else if (label.equalsIgnoreCase("Exit and Discard")) {
		        selectedExitDiscard();
		        return true;
		    } else if (label.equalsIgnoreCase("Exit")) {
		        selectedExit();
		        return true;
			} else 
			
			if (label.equalsIgnoreCase("New...")) {
				selectedNewProject();
				return true;
		    } else if (label.equalsIgnoreCase("Open...")) {
				selectedOpenProject();
				return true;
			} else if (label.equalsIgnoreCase("Edit...")) {
				selectedEditProject();
				return true;
			} else if (label.equalsIgnoreCase("Close")) {
				selectedCloseProject();
				return true;
			} else if (label.equalsIgnoreCase("Module list")) {
				selectedModuleList();
				return true;
		    } else 
		    
		    if (label.equalsIgnoreCase("Contents")) {
				selectedContents();
				return true;
		    } else if (label.equalsIgnoreCase("Java API")) {
				selectedJavaAPI();
				return true;
		    } else if (label.equalsIgnoreCase("About...")) {
				selectedAbout();
				return true;
			} else {
				selectedNewModule(label);
			}

		}
		return super.action(event, arg);
    }
//////////////////////////////////////////////////////////////////
//
//	
//
//////////////////////////////////////////////////////////////////
	public void setProjectActive(){
		menuOpen.disable();
		menuNew.disable();
		menuEdit.enable();
		menuClose.enable();
		moduleMenu.enable();
	}
	public void setProjectInactive(){
		menuOpen.enable();
		menuNew.enable();
		menuEdit.disable();
		menuClose.disable();
		moduleMenu.disable();
		
		if(jaNetModuleList != null)jaNetModuleList.hide();
	}
//////////////////////////////////////////////////////////////////
//
//	Confirmable interface implementors
//
//////////////////////////////////////////////////////////////////
	public void confirmIsYes(int id){
		switch(id){
			case QUIT_CONFIRM:
				exit();
				break;
			case QUIT_SAVEALL_CONFIRM:
				if(currentProject != null){
					try{
						currentProject.save();
					}catch(IOException ioe){
						todo = ""+ioe;
					}
					currentProject.close();
					currentProject = null;
					setProjectInactive();
				}
				exit();
				break;
			case QUIT_ASK_CONFIRM:
				if(currentProject != null){
					if(currentProject.needSave()){
						jaNetConfirmDialog = new confirmDialog(this, "Save project before quit ?", QUIT_SAVEALL_CONFIRM);
						jaNetConfirmDialog.show();
						return;
					}
					currentProject.close();
					currentProject = null;
					setProjectInactive();
				}
				setProjectInactive();
				exit();
				break;
			case PRJ_SAVE:
				try{
					currentProject.save();
				}catch(IOException ioe){
					todo = ""+ioe;
				}
				currentProject.close();
				currentProject = null;
				setProjectInactive();
				break;
		}

	}
	public void confirmIsNo(int id){
		switch(id){
			case PRJ_SAVE:
				currentProject.close();
				currentProject = null;
				setProjectInactive();
				break;
			case QUIT_SAVEALL_CONFIRM:
				currentProject.close();
				currentProject = null;
				setProjectInactive();
				exit();
				break;
		}
		
	}
//////////////////////////////////////////////////////////////////
//
//	menu's selection methods
//
//////////////////////////////////////////////////////////////////
	public void selectedNewModule(String label){
		NNeMo	tempNNeMo;
		// may be a new module ??
		for(int j=0; j< moduleTypeTable.size(); j++){
			ModuleTypes mt = (ModuleTypes)moduleTypeTable.elementAt(j);
			if (label.equalsIgnoreCase( mt.ModuleTypeName)){
				// we found it, try to load and start it
				String className = mt.ModuleClassName;
				try{
					// try to load and create a new instance of a class which the name is specified in fnclassname string.
					tempNNeMo = (NNeMo) Class.forName(className).newInstance();
				}catch (InstantiationException instex){
					todo = "Error in instantiation "+className+".class ("+instex+").";
					return;
				}catch (IllegalAccessException illaccex){
					todo = "Error in accessing "+className+".class ("+illaccex+").";
					return;
				}catch (ClassNotFoundException classnotfound){
					todo = "Error, "+className+".class not found ("+classnotfound+").";
					return;
				}catch(IncompatibleClassChangeError icce){
					todo = ""+icce;
					return;
				}
				// now run it
				ModuleEntry me = new ModuleEntry(currentProject.pathName, null , mt, tempNNeMo);
				currentProject.addNewModuleEntry(me);
				tempNNeMo.NNeMoInit(me);
				return;
			}
		}
		return;
	}
	public void selectedOptions() {
		if(jaNetOptionsBox == null)jaNetOptionsBox = new OptionsBox(this);
		jaNetOptionsBox.show(moduleTypeTable);
	}
	public void selectedExitSaveAll() {
		jaNetConfirmDialog = new confirmDialog(this, "Are You sure to quit jaNet ?", QUIT_SAVEALL_CONFIRM);
		jaNetConfirmDialog.show();
	}
	public void selectedExitDiscard() {
		jaNetConfirmDialog = new confirmDialog(this, "Are You sure to quit jaNet ?", QUIT_CONFIRM);
		jaNetConfirmDialog.show();
	}
    public void selectedExit() {
		jaNetConfirmDialog = new confirmDialog(this, "Are You sure to quit jaNet ?", QUIT_ASK_CONFIRM);
		jaNetConfirmDialog.show();
    }
	public void selectedNewProject() {
		if(currentProject != null){
			todo = "Close project before creating a new.";
			return;
		}
		FilenameFilter filter = new EndsWithFilter("prj");
		jaNetProject tempPrj = null;;
		String path, file;
		
		if(createProjectFD == null) createProjectFD = new FileDialog(this, "Create Project",FileDialog.SAVE);
		createProjectFD.setFilenameFilter(filter);
		createProjectFD.show();
		
		file = createProjectFD.getFile();
		path = createProjectFD.getDirectory();
		
		// clicked Cancel ?
		if(file==null || path==null) return;
		if(file.endsWith("*.*")) file = file.substring(file.length()-3,file.length());
		// do it remains something ?
		if(file.length() == 0) return;
		
		try{
			tempPrj = new jaNetProject(path,file);
			tempPrj.save();
		}catch(IOException ioe){
			todo = "selectedNewProject: "+ioe;
			return;
		}
		currentProject = tempPrj;
		setProjectActive();
	}
	public void selectedOpenProject() {
		if(currentProject != null){
			todo = "Close project before opening a new.";
			return;
		}
		FilenameFilter filter = new EndsWithFilter("prj");
		jaNetProject tempPrj = null;;
		String path, file;
		
		if(openProjectFD == null) openProjectFD = new FileDialog(this, "Open Project",FileDialog.LOAD);
		openProjectFD.setFilenameFilter(filter);
		openProjectFD.show();
		
		file = openProjectFD.getFile();
		path = openProjectFD.getDirectory();
		
		// clicked Cancel ?
		if(file==null || path==null) return;
		if(file.endsWith("*.*")) file = file.substring(file.length()-3,file.length());
		try{
			tempPrj = new jaNetProject(path,file);
			tempPrj.load();
		}catch(IOException ioe){
			System.out.println(""+ioe);
			return;
		}
		currentProject = tempPrj;
		setProjectActive();
	}
	public void selectedEditProject() {
		todo = "Sorry, not yet implemented !";
		// to do: put event handler code here.
	}
	public void selectedCloseProject() {
		if(currentProject == null) return;
		
		if(currentProject.needSave()){
			jaNetConfirmDialog = new confirmDialog(this, "Save project before close ?", PRJ_SAVE);
			jaNetConfirmDialog.show();
			return;
		}
		currentProject.close();
		currentProject = null;
		setProjectInactive();
	}
	public void selectedModuleList() {
		if (jaNetModuleList == null) jaNetModuleList = new ModuleList();
		jaNetModuleList.show(currentProject);
	}
	public void selectedContents() {
		// to do: put event handler code here.
	}
	public void selectedJavaAPI() {
		try{
			Runtime.getRuntime().exec("/bin/sh -c \"netscape "+homePage+"\"");
		}catch(IOException ioe){
			//System.out.println(""+ioe);
		}
	}
    public void selectedAbout() {
		if(jaNetAboutBox == null) jaNetAboutBox = new AboutBox(this, jaNetHomePath);
		jaNetAboutBox.show();
	}
//////////////////////////////////////////////////////////////////
//
//	Read and save configuration file
//
//////////////////////////////////////////////////////////////////
	public void readConfigFile(String path, String name) {
		try {
			System.out.println(path+name);
			File jaNetConfig = new File(path, name);
			RandomAccessFile file = new RandomAccessFile(jaNetConfig,"r");

			readConfigFile(file);
			file.close();
		}
		catch (IOException ioe) {
			System.out.println(ioe);
		}
		catch(SecurityException se){
			System.out.println(se);
		}
	}
	public void readConfigFile(RandomAccessFile file) throws IOException {
		
		// read header
		if(file.readUTF().compareTo(getClass().getName()+version) != 0){
			throw new IOException("jaNet: Error in readConfigFile, unknown version.");
		}
		// read number of modules in file
		int nModules = file.readInt();
		ModuleTypes modTemp;
		
		for(int i=0; i< nModules; i++){
			modTemp = new ModuleTypes(file);
			moduleTypeTable.addElement(modTemp);
        }
    }
	public void writeConfigFile(String path, String name) {
		try {
			File jaNetConfig = new File(path, name);
			RandomAccessFile file = new RandomAccessFile(jaNetConfig,"rw");

			writeConfigFile(file);
			file.close();
		}
		catch (IOException ioe) {
			todo = ""+ioe;
		}
		catch(SecurityException se){
			todo = ""+se;
		}
	}
	public void writeConfigFile(RandomAccessFile file) throws IOException {
		// write "jaNet1.0"
		file.writeUTF(getClass().getName()+version);
		// write the number of elements in the hash table
		file.writeInt(moduleTypeTable.size());
		// write each module type in the config file
		for(int i=0; i< moduleTypeTable.size(); i++){
			((ModuleTypes)moduleTypeTable.elementAt(i)).writeModuleType(file);
		}
    }
//////////////////////////////////////////////////////////////////
//
//	thread guard starter, stopper and body
//
//////////////////////////////////////////////////////////////////
	public void start() { 
	  if(threadGuard == null){
			threadGuard = new Thread(this, "jaNet guard");
			threadGuard.start();
		}
	}

	public void stop(){
		threadGuard.stop();
		threadGuard = null;
	}
	
	public void run(){
		while(true){
			if(todo.compareTo(NOTHING) != 0){
				errorDialog jaNeterrorDialog = new errorDialog(this,todo);
				jaNeterrorDialog.show();
				
				todo = NOTHING;
			}
			// never try to free some memory ...
			System.gc();
			// wait a tenth of a second, then test it again!
			try {Thread.sleep(500);} catch (InterruptedException e){};
		}
	}
  
}
